#!/usr/bin/env python3
"""
Client for Reactome REST API.
https://reactome.org/ContentService/
https://reactome.org/ContentService/data/pathways/top/9606
"""
import sys,os,re,argparse,time,json,logging
#
from .. import reactome
#
##############################################################################
if __name__=='__main__':
  parser = argparse.ArgumentParser(
        description='Reactome (www.reactome.org) REST API client utility',
        epilog="""\
Example IDs:
	1640170 (dbId for "Cell Cycle")
	R-HSA-1640170 (stId for "Cell Cycle")
	109581 (Pathway),
	57033 (EntityWithAccessionedSequence);
	Q13501 (interactor accession (UniProt))
Classes:
	ExternalOntology:Disease,
	Event:Pathway,
	Event:ReactionlikeEvent,
	Person,
	PhysicalEntity [388758],
	PhysicalEntity:Complex [75208],
	PhysicalEntity:Polymer [1181],
	PhysicalEntity:EntitySet [65039],
	PhysicalEntity:GenomeEncodedEntity [244344],
	PhysicalEntity:GenomeEncodedEntity:EntityWithAccessionedSequence [239613],
	PhysicalEntity:SimpleEntity [2668],
	Publication,
	Taxon:Species
""")
  ops = [
          'dbinfo',
          'list_diseases',
          'list_toplevelpathways',
          'list_compounds',
          'query_entry',
          'get_interactors',
          'get_pathwaysforgenes',
          'get_pathwaysforentities',
          'get_pathwayparticipants']
  parser.add_argument("op",choices=ops, help='OPERATION')
  parser.add_argument("--ids", dest="ids", help="IDs, comma-separated")
  parser.add_argument("--i", dest="ifile", help="input file, IDs")
  parser.add_argument("--o", dest="ofile", help="output file")
  parser.add_argument("--api_host", default=reactome.API_HOST)
  parser.add_argument("--api_base_path", default=reactome.API_BASE_PATH)
  parser.add_argument("-v", "--verbose", action="count", default=0)
  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  base_url='https://'+args.api_host+args.api_base_path

  fout = open(args.ofile,"w") if args.ofile else sys.stdout

  t0=time.time()

  if args.ifile:
    fin = open(args.ifile)
    ids = [];
    while True:
      line = fin.readline()
      if not line: break
      if line.rstrip(): ids.append(line.rstrip())
    logging.info('Input queries: {}'.format(len(ids)))
    fin.close()
  elif args.ids:
    ids = re.split('[, ]+', args.ids.strip())

  if args.op == "dbinfo":
    reactome.DBInfo(base_url, fout)

  elif args.op == "query_entry":
    reactome.QueryEntry(ids, base_url, fout)

  elif args.op == "list_diseases":
    reactome.ListDiseases(base_url, fout)

  elif args.op == "list_toplevelpathways":
    reactome.ListToplevelPathways(base_url, fout)

  elif args.op == "list_compounds":
    reactome.ListCompounds(base_url, fout)

  elif args.op == "get_interactors":
    reactome.GetInteractors(ids, base_url, fout)

  elif args.op == "get_pathwaysforgenes":
    reactome.GetPathwaysForGenes(ids, base_url, fout)

  elif args.op == "get_pathwayparticipants":
    reactome.GetPathwayParticipants(ids, base_url, fout)

  else:
    parser.error("No operation specified.")

  logging.info(('elapsed time: %s'%(time.strftime('%Hh:%Mm:%Ss',time.gmtime(time.time()-t0)))))



#!/usr/bin/env python3
###

import sys,os,time,argparse,logging
import numpy as np
import pandas as pd
import h5py

from ...util import hdf
from ... import maayanlab

#############################################################################
if __name__=='__main__':
  parser = argparse.ArgumentParser(description='H5 file operations', epilog="")
  OPS = ['summary', 'list_samples']
  parser.add_argument("op", choices=OPS, help='OPERATION')
  parser.add_argument("--i", dest="ifile", required=True, help="input file")
  parser.add_argument("--o", dest="ofile", help="output (TSV)")
  parser.add_argument("-v", "--verbose", dest="verbose", action="count", default=0)
  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  t0 = time.time();

  fout = open(args.ofile, "w") if args.ofile else sys.stdout

  finh5 = h5py.File(args.ifile, 'r')

  if args.op == "summary":
    logging.debug(list(finh5.keys()))
    hdf.Utils.Summary(finh5)

  elif args.op == "list_samples":
    maayanlab.archs4.ListSamples(finh5, fout)

  else:
    parser.error(f"Unsupported operation: {args.op}")

  logging.info(f"Elapsed: {time.strftime('%Hh:%Mm:%Ss', time.gmtime(time.time()-t0))}")




#!/usr/bin/env python3
"""
Utility for STRING Db REST API.

STRING = Search Tool for the Retrieval of Interacting Genes/Proteins

See: http://string-db.org/help/api/

http://[database]/[access]/[format]/[request]?[parameter]=[value]

database:
string-db.org
string.embl.de
stitch.embl.de
"""
###
import sys,os,re,json,time,logging
import pandas as pd
import urllib,urllib.request,urllib.parse
#
from ..util import rest
#
API_HOST='string-db.org'
API_BASE_PATH='/api'
BASE_URL='https://'+API_HOST+API_BASE_PATH
#
NETWORK_FLAVORS=['evidence', 'confidence', 'actions']
IMG_FMTS=['image', 'highres_image', 'svg']
#
##############################################################################
def GetIds(ids, base_url=BASE_URL, fout=None):
  tags=[]; df=pd.DataFrame();
  for id_this in ids:
    url_this = base_url+f'/json/get_string_ids?identifier={id_this}'
    results = rest.GetURL(url_this, parse_json=True)
    for result in results:
      logging.debug(result)
      if not tags: tags = list(result.keys())
      df = pd.concat([df, pd.DataFrame({tags[j]:[result[tags[j]]] for j in range(len(tags))})])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info('queries: {} ; results: {}'.format(len(ids), df.shape[0]))
  return df

##############################################################################
def GetInteractionPartners(ids, species, minscore, base_url=BASE_URL, fout=None):
  tags=[]; df=pd.DataFrame();
  for id_this in ids:
    url_this = base_url+f'/json/interaction_partners?identifier={id_this}'
    if species: url_this+=(f'&species={species}')
    if minscore: url_this+=(f'&required_score={minscore}')
    results = rest.GetURL(url_this, parse_json=True)
    for result in results:
      logging.debug(result)
      if not tags: tags = list(result.keys())
      df = pd.concat([df, pd.DataFrame({tags[j]:[result[tags[j]]] for j in range(len(tags))})])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info('queries: {} ; interaction partners: {}'.format(len(ids), df.shape[0]))
  return df

##############################################################################
def GetEnrichment(ids, species, minscore, base_url=BASE_URL, fout=None):
  tags=[]; df=pd.DataFrame();
  url = base_url+'/json/enrichment?identifiers={}'.format(urllib.parse.quote('\n'.join(ids)))
  if species: url+=(f'&species={species}')
  if minscore: url+=(f'&required_score={minscore}')
  results = rest.GetURL(url, parse_json=True)
  for result in results:
    logging.debug(result)
    if not tags: tags = list(result.keys())
    df = pd.concat([df, pd.DataFrame({tags[j]:[result[tags[j]]] for j in range(len(tags))})])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info('queries: {} ; enrichment results: {}'.format(len(ids), df.shape[0]))
  return df

##############################################################################
def GetPPIEnrichment(ids, species, minscore, base_url=BASE_URL, fout=None):
  tags=[]; df=pd.DataFrame();
  url = base_url+'/json/ppi_enrichment?identifiers={}'.format(urllib.parse.quote('\n'.join(ids)))
  if species: url+=(f'&species={species}')
  if minscore: url+=(f'&required_score={minscore}')
  results = rest.GetURL(url, parse_json=True)
  for result in results:
    logging.debug(result)
    if not tags: tags = list(result.keys())
    df = pd.concat([df, pd.DataFrame({tags[j]:[result[tags[j]]] for j in range(len(tags))})])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info('queries: {} ; enrichment results: {}'.format(len(ids), df.shape[0]))
  return df

##############################################################################
def GetNetwork(nid, species, minscore, netflavor, base_url=BASE_URL, fout=None):
  tags=[]; df=pd.DataFrame();
  url = base_url+f'/json/network?identifier={nid}'
  if species: url+=(f'&species={species}')
  if minscore: url+=(f'&required_score={minscore}')
  if netflavor: url+=(f'&network_flavor={netflavor}')
  edges = rest.GetURL(url, parse_json=True)
  logging.debug(json.dumps(edges, indent=2))
  for edge in edges:
    logging.debug(edge)
    if not tags: tags = list(edge.keys())
    df = pd.concat([df, pd.DataFrame({tags[j]:[edge[tags[j]]] for j in range(len(tags))})])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info('edges: {}'.format(df.shape[0]))
  return df

##############################################################################
def GetNetworkImage(nid, species, minscore, netflavor, imgfmt, base_url=BASE_URL):
  url = base_url+f'/{imgfmt}/network?identifier={nid}'
  if species: url+=(f'&species={species}')
  if minscore: url+=(f'&required_score={minscore}')
  if netflavor: url+=(f'&network_flavor={netflavor}')
  img = rest.GetURL(url, parse_json=False, parse_xml=False)
  return img

##############################################################################
def GetInteractors(pids, species, minscore, base_url=BASE_URL, fout=None):
  """2018: May be deprecated since 2015."""
  tags=[]; df=pd.DataFrame();
  url=base_url+'/tsv/interactors'
  if len(pids)==1:
    url+=('?identifier=%s'%pids[0])
  else:
    url+=('?identifiers=%s'%urllib.parse.quote('\n'.join(pids)))
  url+=('&species=%s'%species if species else '')
  url+=('&required_score=%d'%minscore)
  rval=rest.GetURL(url, parse_xml=False, parse_json=False)
  fout.write(rval)
  logging.info('queries: %d ; interactors: %d'%(len(pids), len(rval.splitlines())-1))

##############################################################################
def GetActions(pids, species, minscore, base_url=BASE_URL, fout=None):
  """2018: May be deprecated since 2015."""
  tags=[]; df=pd.DataFrame();
  url=base_url+'/tsv/actions'
  if len(pids)==1:
    url+=('?identifier=%s'%pids[0])
  else:
    url+=('?identifiers=%s'%urllib.parse.quote('\n'.join(pids)))
  url+=('&species=%s'%species if species else '')
  url+=('&required_score=%d'%minscore)
  rval=rest.GetURL(url, parse_xml=False, parse_json=False)
  fout.write(rval)
  logging.info('queries: %d ; actions: %d'%(len(pids), len(rval.splitlines())-1))

##############################################################################
def GetAbstracts(pids, species, base_url=BASE_URL, fout=None):
  """2018: May be deprecated since 2015."""
  tags=[]; df=pd.DataFrame();
  url=base_url+'/tsv/abstracts'
  if len(pids)==1:
    url+=('?identifier=%s'%pids[0])
  else:
    url+=('?identifiers=%s'%urllib.parse.quote('\n'.join(pids)))
  url+=('&species=%s'%species if species else '')
  rval=rest.GetURL(url, parse_xml=False, parse_json=False)
  fout.write(rval)
  logging.info('queries: %d ; abstracts: %d'%(len(pids), len(rval.splitlines())-1))

##############################################################################



#!/usr/bin/env python3
"""
DrugCentral db utility functions.
"""
import os,sys,re,json,logging,yaml,tqdm
import pandas as pd
import psycopg2,psycopg2.extras

NCHUNK=100

#############################################################################
def Connect(dbhost, dbport, dbname, dbusr, dbpw):
  """Connect to db; specify default cursor type DictCursor."""
  dsn = (f"host='{dbhost}' port='{dbport}' dbname='{dbname}' user='{dbusr}' password='{dbpw}'")
  dbcon = psycopg2.connect(dsn)
  dbcon.cursor_factory = psycopg2.extras.DictCursor
  return dbcon

#############################################################################
def Version(dbcon, dbschema="public", fout=None):
  sql = (f"SELECT * FROM {dbschema}.dbversion")
  logging.debug(f"SQL: {sql}")
  df = pd.read_sql(sql, dbcon)
  if fout: df.to_csv(fout, "\t", index=False)
  return df

#############################################################################
def MetaListdbs(dbcon, fout=None):
  """Pg meta-command: list dbs from pg_database."""
  sql = ("SELECT pg_database.datname, pg_shdescription.description FROM pg_database LEFT OUTER JOIN pg_shdescription on pg_shdescription.objoid = pg_database.oid WHERE pg_database.datname ~ '^drug'")
  logging.debug(f"SQL: {sql}")
  df = pd.read_sql(sql, dbcon)
  if fout: df.to_csv(fout, "\t", index=False)
  return df

#############################################################################
def ListColumns(dbcon, dbschema="public", fout=None):
  df=None;
  sql1 = (f"SELECT table_name FROM information_schema.tables WHERE table_schema = '{dbschema}'")
  df1 = pd.read_sql(sql1, dbcon)
  for tname in df1.table_name:
    sql2 = (f"SELECT column_name,data_type FROM information_schema.columns WHERE table_schema = '{dbschema}' AND table_name = '{tname}'")
    df_this = pd.read_sql(sql2, dbcon)
    df_this["schema"] = dbschema
    df_this["table"] = tname
    df = df_this if df is None else pd.concat([df, df_this])
  df = df[["schema", "table", "column_name", "data_type"]]
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_out: {df.shape[0]}")
  return df

#############################################################################
def ListTables(dbcon, dbschema="public", fout=None):
  '''Listing the tables.'''
  sql = (f"SELECT table_name FROM information_schema.tables WHERE table_schema = '{dbschema}'")
  df = pd.read_sql(sql, dbcon)
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_out: {df.shape[0]}")
  return df

#############################################################################
def ListTablesRowCounts(dbcon, dbschema="public", fout=None):
  '''Listing the table rowcounts.'''
  df=None;
  sql1 = (f"SELECT table_name FROM information_schema.tables WHERE table_schema = '{dbschema}'")
  df1 = pd.read_sql(sql1, dbcon)
  for tname in df1.table_name:
    sql2 = (f"SELECT COUNT(*) AS rowcount FROM {dbschema}.{tname}")
    df_this = pd.read_sql(sql2, dbcon)
    df_this["schema"] = dbschema
    df_this["table"] = tname
    df = df_this if df is None else pd.concat([df, df_this])
  df = df[["schema", "table", "rowcount"]]
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_out: {df.shape[0]}")
  return df

#############################################################################
def ListStructures(dbcon, dbschema="public", fout=None):
  sql = (f"SELECT id,name,cas_reg_no,smiles,inchikey,inchi,cd_formula AS formula,cd_molweight AS molweight FROM {dbschema}.structures")
  logging.debug(f"SQL: {sql}")
  df = pd.read_sql(sql, dbcon)
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_out: {df.shape[0]}")
  return df

#############################################################################
def ListStructures2Smiles(dbcon, dbschema="public", fout=None):
  sql = (f"SELECT smiles, id, name FROM {dbschema}.structures WHERE smiles IS NOT NULL")
  logging.debug(f"SQL: {sql}")
  df = pd.read_sql(sql, dbcon)
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_out: {df.shape[0]}")
  return df

#############################################################################
def ListStructures2Xref(dbcon, xref_type, dbschema="public", fout=None):
  df=None; n_out=0;
  sql = f"""\
SELECT
	structures.id struct_id,
	structures.smiles,
	structures.name dc_struct_name,
	identifier.identifier {xref_type.lower()}
FROM
	structures
	JOIN identifier ON identifier.struct_id = structures.id 
WHERE
	identifier.id_type = '{xref_type}'
"""
  logging.debug(f"SQL: {sql}")
  df_itr = pd.read_sql(sql, dbcon, chunksize=NCHUNK)
  for df_this in df_itr:
    if fout is not None: df_this.to_csv(fout, "\t", index=False)
    else: df = pd.concat([df, df_this])
    n_out += df_this.shape[0]
  logging.info(f"n_out: {n_out}")
  return df

#############################################################################
def ListStructures2Pubchem(dbcon, dbschema="public", fout=None):
  return ListStructures2Xref(dbcon, "PUBCHEM_CID", dbschema, fout)

#############################################################################
def ListStructures2Chembl(dbcon, dbschema="public", fout=None):
  return ListStructures2Xref(dbcon, "ChEMBL_ID", dbschema, fout)

#############################################################################
def ListStructures2Molfile(dbcon, dbschema="public", fout=None):
  """fout is required arg."""
  n_out=0;
  cur = dbcon.cursor()
  cur.execute(f"SELECT molfile, id, name FROM {dbschema}.structures WHERE molfile IS NOT NULL")
  for row in cur:
    molfile = re.sub(r'^[\s]*\n', str(row["id"])+"\n", row["molfile"])
    fout.write(molfile)
    fout.write("> <DRUGCENTRAL_STRUCT_ID>\n"+str(row["id"])+"\n\n")
    fout.write("> <NAME>\n"+row["name"]+"\n\n")
    fout.write("$$$$\n")
    n_out+=1
  logging.info(f"n_out: {n_out}")

#############################################################################
def ListProducts(dbcon, dbschema="public", fout=None):
  sql = (f"SELECT * FROM {dbschema}.product")
  logging.debug(f"SQL: {sql}")
  df = pd.read_sql(sql, dbcon)
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_out: {df.shape[0]}")
  return df

#############################################################################
def ListActiveIngredients(dbcon, dbschema="public", fout=None):
  sql = (f"SELECT * FROM {dbschema}.active_ingredient")
  logging.debug(f"SQL: {sql}")
  df = pd.read_sql(sql, dbcon)
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_out: {df.shape[0]}")
  return df

#############################################################################
def ListXrefTypes(dbcon, fout=None):
  sql="""\
SELECT DISTINCT
	id AS xref_type_id,
	type AS xref_type,
	description AS xref_type_description
FROM
	id_type
ORDER BY
	xref_type
"""
  logging.debug(f"SQL: {sql}")
  df = pd.read_sql(sql, dbcon)
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_out: {df.shape[0]}")
  return df

#############################################################################
def ListXrefs(dbcon, fout=None):
  df=None; n_out=0; tq=None;
  quiet = bool(logging.getLogger().getEffectiveLevel()>15)
  N_row = pd.read_sql("SELECT COUNT(*) FROM identifier", dbcon).iloc[0,0]
  sql="""\
SELECT
	identifier.struct_id,
	identifier.id_type xref_type,
	identifier.identifier xref,
	s.name dc_struct_name
FROM
	identifier
	JOIN structures s ON s.id=identifier.struct_id
"""
  df_itr = pd.read_sql(sql, dbcon, chunksize=NCHUNK)
  for df_this in df_itr:
    if not quiet and tq is None: tq = tqdm.tqdm(total=N_row)
    if fout is not None: df_this.to_csv(fout, "\t", header=bool(n_out==0), index=False)
    else: df = pd.concat([df, df_this])
    if tq is not None: tq.update(df_this.shape[0])
    n_out += df_this.shape[0]
  if tq is not None: tq.close()
  logging.info(f"rows: {n_out}")
  return df

#############################################################################
def ListIndications(dbcon, fout=None):
  sql="""\
SELECT DISTINCT
	omop.concept_id omop_concept_id,
	omop.concept_name omop_concept_name,
	omop.umls_cui,
	omop.cui_semantic_type umls_semantic_type,
	omop.snomed_conceptid,
	omop.snomed_full_name
FROM
	omop_relationship omop
JOIN
	structures s ON omop.struct_id = s.id
WHERE
	omop.relationship_name = 'indication'
"""
  logging.debug(f"SQL: {sql}")
  df = pd.read_sql(sql, dbcon)
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_out: {df.shape[0]}")
  return df

#############################################################################
def ListIndicationTargets(dbcon, fout=None):
  sql="""\
SELECT DISTINCT
	omop.concept_name AS omop_concept,
	omop.umls_cui,
	atf.struct_id,
	s.name AS struct_name,
	atf.target_id,
	atf.target_name,
	atf.gene,
	atf.action_type,
	atf.act_source,
	atf.act_type,
	atf.act_comment,
	atf.relation,
	atf.moa,
	atf.moa_source,
	atf.moa_source_url,
	r.pmid AS ref_pmid,
	r.doi AS ref_doi,
	r.title AS ref_title,
	r.dp_year AS ref_year
FROM
	act_table_full atf
	JOIN structures s ON s.id = atf.struct_id
	JOIN omop_relationship omop ON omop.struct_id = s.id
	LEFT OUTER JOIN reference r ON r.id = atf.moa_ref_id
WHERE
	omop.relationship_name = 'indication'
"""
  logging.debug(f"SQL: {sql}")
  df = pd.read_sql(sql, dbcon)
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_out: {df.shape[0]}")
  return df

#############################################################################
def ListDrugdruginteractions(dbcon, fout=None):
  sql="""\
SELECT
	ddi.id AS ddi_id,
	ddi.drug_class1,
	ddi.drug_class2,
	ddi.source_id,
	drug_class1.id drug_class_id1,
	drug_class1.source source1,
	drug_class1.is_group is_group1,
	drug_class2.id drug_class_id2,
	drug_class2.source source2,
	drug_class2.is_group is_group2
FROM
	ddi
JOIN drug_class drug_class1 ON drug_class1.name = ddi.drug_class1
JOIN drug_class drug_class2 ON drug_class2.name = ddi.drug_class2
"""
  logging.debug(f"SQL: {sql}")
  df = pd.read_sql(sql, dbcon)
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_out: {df.shape[0]}")
  return df

#############################################################################
def SearchProducts(dbcon, terms, fout=None):
  """Search names via Pg regular expression (SIMILAR TO)."""
  df=None;
  for term in terms:
    sql= f"""\
SELECT DISTINCT
	s.id struct_id,
	s.name struct_name,
	p.id product_id,
	p.ndc_product_code,
	p.form product_form,
	p.generic_name product_generic_name,
	p.product_name,
	p.route product_route
FROM
	structures AS s
JOIN 
	active_ingredient ai ON ai.struct_id = s.id
JOIN 
	product p ON p.ndc_product_code = ai.ndc_product_code
WHERE
	p.product_name.name ~* '{term}' OR p.generic_name product_generic_name ~* '{term}' 
"""
    logging.debug(f"SQL: {sql}")
    df_this = pd.read_sql(sql, dbcon)
    df = pd.concat([df, df_this])
  logging.info(f"n_out: {df.shape[0]}")
  if fout is not None: df.to_csv(fout, "\t", index=False)
  else: return df

#############################################################################
def SearchIndications(dbcon, terms, fout=None):
  """Search names via Pg regular expression (SIMILAR TO)."""
  df=None;
  for term in terms:
    sql=f"""\
SELECT DISTINCT
	omop.concept_id omop_concept_id,
	omop.concept_name omop_concept_name,
	omop.umls_cui,
	omop.cui_semantic_type umls_semantic_type,
	omop.snomed_conceptid,
	omop.snomed_full_name
FROM
	omop_relationship omop
JOIN
	structures s ON omop.struct_id = s.id
WHERE
	omop.relationship_name = 'indication'
	AND (omop.concept_name ~* '{term}' OR omop.snomed_full_name ~* '{term}')
"""
    logging.debug(f"SQL: {sql}")
    df_this = pd.read_sql(sql, dbcon)
    df = df_this if df is None else pd.concat([df, df_this])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_out: {df.shape[0]}")
  return df

#############################################################################
def GetIndicationStructures(dbcon, ids, fout=None):
  """Input OMOP conceptIds (INTEGER)."""
  df=None;
  sql="""\
SELECT DISTINCT
	omop.concept_id omop_concept_id,
	omop.concept_name omop_concept_name,
	s.id struct_id,
	s.name struct_name,
	s.smiles,
	s.inchikey,
	s.inchi,
	s.cd_formula,
	s.cd_molweight
FROM
	omop_relationship omop
JOIN
	structures s ON omop.struct_id = s.id
WHERE
	omop.relationship_name = 'indication'
	AND omop.concept_id = {}
"""
  for id_this in ids:
    logging.debug(sql.format(id_this))
    df_this = pd.read_sql(sql.format(id_this), dbcon)
    df = df_this if df is None else pd.concat([df, df_this])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_out: {df.shape[0]}")
  return df

#############################################################################
def GetStructure(dbcon, ids, fout=None):
  df=None;
  sql = ("""SELECT id,name,cas_reg_no,smiles,inchikey,inchi,cd_formula AS formula,cd_molweight AS molweight FROM structures WHERE id = '{}'""")
  for id_this in ids:
    logging.debug(sql.format(id_this))
    df_this = pd.read_sql(sql.format(id_this), dbcon)
    df = df_this if df is None else pd.concat([df, df_this])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_out: {df.shape[0]}")
  return df

#############################################################################
def GetDrugPage(dbcon, struct_id, fout):
  """Structure, with IDs, names, xrefs, and ATCs, plus associated products."""
  df_struct = GetStructure(dbcon, [struct_id], None) # Should return one row.
  if df_struct.empty: return None
  drug = df_struct.to_dict(orient='records')[0]

  #Add xrefs
  df_xrefs = GetStructureXrefs(dbcon, [struct_id], None)
  drug["xrefs"] = df_xrefs[["xref_type", "xref"]].to_dict(orient='records')

  #Add names
  df_names = GetStructureSynonyms(dbcon, [struct_id], None)
  drug["synonyms"] = df_names["synonym"].tolist()

  #Add ATCs 
  df_atcs = GetStructureAtcs(dbcon, [struct_id], None)
  if not df_atcs.empty:
    drug["atcs"] = df_atcs[["atc_code","atc_l1_code","atc_l1_name","atc_l2_code","atc_l2_name","atc_l3_code","atc_l3_name","atc_l4_code","atc_l4_name","atc_substance"]].to_dict(orient='records')

  #Add products 
  df_products = GetStructureProducts(dbcon, [struct_id], None)
  if not df_products.empty:
    drug["products"] = df_products[["product_id","ndc_product_code","product_form","product_generic_name","product_name","product_route","product_marketing_status","product_active_ingredient_count"]].to_dict(orient='records')

  #Add targets 
  df_targets = GetStructureTargets(dbcon, [struct_id], None)
  if not df_targets.empty:
    drug["targets"] = df_targets[["target_id","target_name","gene","action_type","act_source","act_type","act_comment","relation","moa","moa_source","moa_source_url","ref_pmid","ref_doi","ref_title","ref_year"]].to_dict(orient='records')

  fout.write(json.dumps(drug, indent=2))

#############################################################################
def GetStructureByXref(dbcon, ids, xref_type=None, fout=None):
  if not xref_type:
    logging.error("xref_type required.")
    return None
  df=None;
  sql = ("""\
SELECT
	identifier.identifier xref,
	identifier.id_type xref_type,
	s.id dc_struct_id,
	s.name dc_struct_name
FROM
	structures AS s
	JOIN identifier ON identifier.struct_id=s.id
WHERE
	identifier.identifier = '{0}'
""")
  if xref_type is not None:
    sql += f" AND identifier.id_type = '{xref_type}'"
  for id_this in ids:
    logging.debug(sql.format(id_this))
    df_this = pd.read_sql(sql.format(id_this), dbcon)
    df = df_this if df is None else pd.concat([df, df_this])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_out: {df.shape[0]}")
  return df

#############################################################################
def GetStructureBySynonym(dbcon, ids, fout=None):
  df=None;
  sql = ("""SELECT str.id, str.name structure_name, syn.name synonym FROM structures AS str JOIN synonyms AS syn ON syn.id=str.id WHERE syn.name = '{}'""")
  for id_this in ids:
    logging.debug(sql.format(id_this))
    df_this = pd.read_sql(sql.format(id_this), dbcon)
    df = df_this if df is None else pd.concat([df, df_this])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_out: {df.shape[0]}")
  return df

#############################################################################
def GetStructureXrefs(dbcon, ids, fout=None):
  df=None;
  sql = ("""SELECT struct_id, id_type AS xref_type, identifier AS xref FROM identifier WHERE struct_id = '{}'""")
  for id_this in ids:
    logging.debug(sql.format(id_this))
    df_this = pd.read_sql(sql.format(id_this), dbcon)
    df = df_this if df is None else pd.concat([df, df_this])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_out: {df.shape[0]}")
  return df

#############################################################################
def GetStructureProducts(dbcon, ids, fout=None):
  df=None;
  sql="""\
SELECT DISTINCT
	s.id struct_id,
	s.name struct_name,
	s.smiles,
	s.inchikey,
	p.id product_id,
	p.ndc_product_code,
	p.form product_form,
	p.generic_name product_generic_name,
	p.product_name,
	p.route product_route,
	p.marketing_status product_marketing_status,
	p.active_ingredient_count product_active_ingredient_count
FROM
	structures AS s
JOIN 
	active_ingredient ai ON ai.struct_id = s.id
JOIN 
	product p ON p.ndc_product_code = ai.ndc_product_code
WHERE 
	s.id = '{}'
"""
  for id_this in ids:
    logging.debug(sql.format(id_this))
    df_this = pd.read_sql(sql.format(id_this), dbcon)
    df = df_this if df is None else pd.concat([df, df_this])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_out: {df.shape[0]}")
  return df

#############################################################################
def GetStructureOBProducts(dbcon, ids, fout=None):
  df=None;
  sql="""\
SELECT DISTINCT
	s.id struct_id,
	s.name struct_name,
	s.smiles,
	s.inchikey,
	ob.id ob_id,
	ob.product_no ob_product_no,
	ob.ingredient,
	ob.dose_form,
	ob.route,
	ob.strength,
	ob.appl_type,
	ob.appl_no
FROM
	structures s
	JOIN struct2obprod s2ob ON s2ob.struct_id = s.id
	JOIN ob_product ob ON ob.id = s2ob.prod_id
WHERE 
	s.id = '{}'
"""
  for id_this in ids:
    logging.debug(sql.format(id_this))
    df_this = pd.read_sql(sql.format(id_this), dbcon)
    df = df_this if df is None else pd.concat([df, df_this])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_out: {df.shape[0]}")
  return df

#############################################################################
def GetStructureTargets(dbcon, ids, fout=None):
  df=None; n_out=0;
  sql="""\
SELECT DISTINCT
	atf.struct_id,
	atf.target_id,
	atf.target_name,
	atf.gene,
	atf.action_type,
	atf.act_source,
	atf.act_type,
	atf.act_comment,
	atf.relation,
	atf.moa,
	atf.moa_source,
	atf.moa_source_url,
	r.pmid AS ref_pmid,
	r.doi AS ref_doi,
	r.title AS ref_title,
	r.dp_year AS ref_year
FROM
	act_table_full atf
	JOIN structures s ON s.id = atf.struct_id
	LEFT OUTER JOIN reference r ON r.id = atf.moa_ref_id
WHERE
	atf.struct_id = {}
"""
  for id_this in ids:
    logging.debug(sql.format(id_this))
    df_this = pd.read_sql(sql.format(id_this), dbcon)
    if fout is None: df = pd.concat([df, df_this])
    else: df_this.to_csv(fout, "\t", index=False)
    n_out += df_this.shape[0]
  logging.info(f"n_out: {n_out}")
  return df

#############################################################################
def ListAtcs(dbcon, fout=None):
  """List ATC codes and drug count for which drugs exist."""
  sql="""\
SELECT DISTINCT
	atc.l1_code atc_l1_code, atc.l1_name atc_l1_name,
	atc.l2_code atc_l2_code, atc.l2_name atc_l2_name,
	atc.l3_code atc_l3_code, atc.l3_name atc_l3_name,
	atc.l4_code atc_l4_code, atc.l4_name atc_l4_name,
	COUNT(DISTINCT s.id) drug_count
FROM
	atc
JOIN 
	struct2atc ON struct2atc.id = atc.id
JOIN 
	structures s ON s.id = struct2atc.struct_id
GROUP BY
	atc.l1_code, atc.l1_name,
	atc.l2_code, atc.l2_name,
	atc.l3_code, atc.l3_name,
	atc.l4_code, atc.l4_name
ORDER BY
	atc.l1_name, atc.l2_name, atc.l3_name, atc.l4_name
"""
  logging.debug(sql)
  df = pd.read_sql(sql, dbcon)
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_out: {df.shape[0]}")
  return df

#############################################################################
def ListSynonyms(dbcon, fout=None):
  df=None; n_out=0; tq=None;
  quiet = bool(logging.getLogger().getEffectiveLevel()>15)
  N_row = pd.read_sql("SELECT COUNT(*) FROM synonyms", dbcon).iloc[0,0]
  sql="""\
SELECT
	id struct_id,
	synonyms.name AS synonym
FROM
	synonyms
ORDER BY
	id
"""
  df_itr = pd.read_sql(sql, dbcon, chunksize=NCHUNK)
  for df_this in df_itr:
    if not quiet and tq is None: tq = tqdm.tqdm(total=N_row)
    if fout is not None: df_this.to_csv(fout, "\t", header=bool(n_out==0), index=False)
    else: df = pd.concat([df, df_this])
    if tq is not None: tq.update(df_this.shape[0])
    n_out += df_this.shape[0]
  if tq is not None: tq.close()
  logging.info(f"rows: {n_out}")
  return df

#############################################################################
def GetStructureSynonyms(dbcon, ids, fout=None):
  df=None;
  sql="""\
SELECT DISTINCT
	s.id struct_id,
	s.name struct_name,
	synonyms.name AS synonym
FROM
	structures AS s
JOIN 
	synonyms ON synonyms.id = s.id
WHERE 
	s.id = '{}'
"""
  for id_this in ids:
    logging.debug(sql.format(id_this))
    df_this = pd.read_sql(sql.format(id_this), dbcon)
    df = df_this if df is None else pd.concat([df, df_this])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_out: {df.shape[0]}")
  return df

#############################################################################
def GetStructureAtcs(dbcon, ids, fout=None):
  df=None;
  sql="""\
SELECT DISTINCT
	s.id struct_id,
	s.name struct_name,
	s.smiles,
	s.inchikey,
	atc.code AS atc_code,
	atc.l1_code AS atc_l1_code,
	atc.l1_name AS atc_l1_name,
	atc.l2_code AS atc_l2_code,
	atc.l2_name AS atc_l2_name,
	atc.l3_code AS atc_l3_code,
	atc.l3_name AS atc_l3_name,
	atc.l4_code AS atc_l4_code,
	atc.l4_name AS atc_l4_name,
	atc.chemical_substance AS atc_substance
FROM
	structures AS s
JOIN 
	struct2atc ON struct2atc.struct_id = s.id
JOIN 
	atc ON atc.id = struct2atc.id
WHERE 
	s.id = '{}'
"""
  for id_this in ids:
    logging.debug(sql.format(id_this))
    df_this = pd.read_sql(sql.format(id_this), dbcon)
    df = df_this if df is None else pd.concat([df, df_this])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_out: {df.shape[0]}")
  return df

#############################################################################
def GetProductStructures(dbcon, ids, fout=None):
  df=None;
  sql="""\
SELECT DISTINCT
	p.id product_id,
	p.product_name,
	s.id struct_id,
	s.name struct_name,
	s.smiles,
	s.inchikey,
	s.inchi,
	s.cd_formula,
	s.cd_molweight
FROM
	product p
JOIN 
	active_ingredient ai ON p.ndc_product_code = ai.ndc_product_code
JOIN 
	structures s ON ai.struct_id = s.id
WHERE 
	p.id = '{}'
"""
  for id_this in ids:
    logging.debug(sql.format(id_this))
    df_this = pd.read_sql(sql.format(id_this), dbcon)
    df = pd.concat([df, df_this])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_out: {df.shape[0]}")
  return df

#############################################################################



#!/usr/bin/env python3
"""
Utility functions for the CAS REST API.
"""
###
import sys,os,io,re,csv,json,time,logging,tempfile,tqdm
import requests
import urllib.request,urllib.parse
import pandas as pd
#
#
API_HOST='commonchemistry.cas.org'
API_BASE_PATH='/api'
BASE_URL="https://"+API_HOST+API_BASE_PATH
#
# https://commonchemistry.cas.org/api/detail?uri=substance%2Fpt%2F50000
#############################################################################
def GetRN2Details(ids, base_url=BASE_URL, fout=None):
  n_out=0; n_err=0; tags=None; df=None; tq=None;
  for i,id_this in enumerate(ids):
    uri = urllib.parse.quote(f"substance/pt/{id_this}")
    url = (base_url+f"/detail?uri={uri}")
    if tq is None: tq = tqdm.tqdm(total=len(ids), unit="mols")
    tq.update(n=1)
    response = requests.get(url, headers={"Accept": "application/json"})
    logging.debug(response.text)
    if response.status_code==requests.codes.not_found:
      continue
    if response.status_code!=requests.codes.ok:
      logging.error(f"HTTP status_code: {response.status_code}")
    mol = response.json()
    if not tags:
      tags = list(mol.keys())
      for tag in tags[:]:
        if type(mol[tag]) in (dict, list, tuple):
          tags.remove(tag)
          logging.debug(f'Ignoring field: "{tag}"')
      if "image" in tags:
        tags.remove("image")
        logging.debug(f'Ignoring field: "image"')
    synonyms = mol["synonyms"] if "synonyms" in mol else []
    replacedRns = mol["replacedRns"] if "replacedRns" in mol else []
    data_this = {tag:[mol[tag]] for tag in tags}
    data_this["synonyms"] = ",".join(synonyms)
    data_this["replacedRns"] = ",".join(replacedRns)
    df_this = pd.DataFrame(data_this)
    if fout is not None:
      df_this.to_csv(fout, sep='\t', index=False, header=bool(n_out==0))
    else:
      df = pd.concat([df, df_this])
    n_out+=1
  tq.close()
  logging.info(f"Input IDs: {len(ids)}; Output records: {n_out}")
  return df



#!/usr/bin/env python3
"""
Utility app for the PubChem PUG REST API.
"""
###
import sys,os,re,argparse,time,logging
#
from .. import pubchem
#
##############################################################################
if __name__=='__main__':
  OPS = [
        "list_sources_substance", "list_sources_assay",
        "get_name2sid", "get_name2cid", "get_name2synonyms",
        "get_smi2cid",
        "get_cid2smiles", "get_cid2sdf",
        "get_cid2properties", "get_cid2inchi",
        "get_cid2descriptions",
        "get_cid2synonyms", "get_cid2sid", "get_cid2assaysummary",
        "get_cid2nicename",
        "get_sid2cid", "get_sid2sdf", "get_sid2assaysummary",
        "get_assayname", "get_assaydescriptions",
	"get_assaysubstances",
	"get_assaysubstanceresults",]
  parser = argparse.ArgumentParser(description="PubChem PUG REST client")
  parser.add_argument("op", choices=OPS, help="OPERATION")
  parser.add_argument("--i", dest="ifile", help="input IDs file (CID|SID|SMILES|name)")
  parser.add_argument("--ids", help="input IDs (CID|SID|SMILES|name) (comma-separated)")
  parser.add_argument("--aids", help="input AIDs (comma-separated)")
  parser.add_argument("--iaid", dest="ifile_aid", help="input AIDs file")
  parser.add_argument("--o", dest="ofile", help="output (usually TSV)")
  parser.add_argument("--api_host", default=pubchem.API_HOST)
  parser.add_argument("--api_base_path", default=pubchem.API_BASE_PATH)
  parser.add_argument("--skip", type=int, default=0)
  parser.add_argument("--nmax", type=int, default=0)
  parser.add_argument("--nmax_per_cid", type=int, default=20)
  parser.add_argument("-q", "--quiet", action="store_true", help="Suppress progress notification.")
  parser.add_argument("-v", "--verbose", default=0, action="count")
  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>0 else logging.ERROR if args.quiet else 15))

  base_url = f"https://{args.api_host}{args.api_base_path}"

  fout = open(args.ofile, "w") if args.ofile else sys.stdout

  ids=[]
  if args.ifile:
    fin = open(args.ifile)
    while True:
      line = fin.readline()
      if not line: break
      ids.append(line.rstrip())
    fin.close()
  elif args.ids:
    ids = re.split(r'[,\s]+', args.ids)
  logging.info(f"Input IDs: {len(ids)}")

  aids=[]
  if args.ifile_aid:
    fin = open(args.ifile_aid)
    while True:
      line = fin.readline()
      if not line: break
      aids.append(line.rstrip())
    fin.close()
  elif args.aids:
    aids = re.split(r'[,\s]+', args.aids)
  if len(aids)>0: logging.info(f"Input AIDs: {len(aids)}")

  t0=time.time()

  if args.op == 'list_sources_assay':
    pubchem.ListSources("assay", base_url, fout)

  elif args.op == 'list_sources_substance':
    pubchem.ListSources("substance", base_url, fout)

  elif args.op == 'get_cid2synonyms':
    pubchem.GetCID2Synonyms(ids, args.skip, args.nmax, args.nmax_per_cid, base_url, fout)

  elif args.op == 'get_cid2nicename':
    pubchem.GetCID2Nicename(ids, args.skip, args.nmax, base_url, fout)

  elif args.op == 'get_cid2descriptions':
    pubchem.GetCID2Descriptions(ids, base_url, fout)

  elif args.op == 'get_cid2properties':
    pubchem.GetCID2Properties(ids, base_url, fout)

  elif args.op == 'get_cid2inchi':
    pubchem.GetCID2Inchi(ids, base_url, fout)

  elif args.op == 'get_cid2sid':
    pubchem.GetCID2SID(ids, base_url, fout)

  elif args.op == 'get_cid2smiles':
    pubchem.Utils.GetCID2Smiles(ids, base_url, fout)

  elif args.op == 'get_cid2sdf':
    pubchem.GetCID2SDF(ids, base_url, fout)

  elif args.op == 'get_cid2assaysummary':
    pubchem.GetCID2AssaySummary(ids, base_url, fout)

  elif args.op == 'get_sid2cid':
    pubchem.GetSID2CID(ids, base_url, fout)

  elif args.op == 'get_sid2assaysummary':
    pubchem.GetSID2AssaySummary(ids, base_url, fout)

  elif args.op == 'get_sid2sdf':
    pubchem.GetSID2SDF(ids, fout, args.skip, args.nmax, base_url)

  elif args.op == 'get_smi2cid':
    pubchem.GetSmiles2CID(ids, base_url, fout)

  elif args.op == 'get_name2sid':
    pubchem.GetName2SID(ids, args.skip, args.nmax, base_url, fout)

  elif args.op == 'get_name2cid':
    pubchem.GetName2CID(ids, args.skip, args.nmax, base_url, fout)

  elif args.op == 'get_name2synonyms':
    pubchem.GetName2Synonyms(ids, args.skip, args.nmax, base_url, fout)

  elif args.op == 'get_assayname':
    pubchem.GetAssayName(aids, base_url, fout)

  elif args.op == 'get_assaydescriptions':
    pubchem.GetAssayDescriptions(aids, args.skip, args.nmax, base_url, fout)

  elif args.op == 'get_assaysubstances':
    pubchem.Utils.GetAssaySIDs(aids, args.skip, args.nmax, base_url, fout)

  elif args.op == 'get_assaysubstanceresults':
    if not (aids and ids): parser.error('Input AIDs and SIDs required.')
    pubchem.GetAssaySIDResults(aids, ids, args.skip, args.nmax, base_url, fout)

  else:
    parser.error(f"Invalid operation: {args.op}")

  logging.info(('elapsed time: %s'%(time.strftime('%Hh:%Mm:%Ss',time.gmtime(time.time()-t0)))))




#!/usr/bin/env python3
"""
Utility functions for PDB REST API.
See: http://www.rcsb.org/pdb/software/rest.do
API supports CSV output.  So we could use this directly.

Ligand types:
    "D-beta-peptide, C-gamma linking",	20,0.010%
    "D-gamma-peptide, C-delta linking",	39,0.019%
    "D-peptide NH3 amino terminus",	1,0.000%
    "D-peptide linking",		1222,0.588%
    "D-saccharide",			10329,4.967%
    "D-saccharide 1, 4 and 1, 4 linking",	39,0.019%
    "DNA OH 3 prime terminus",		20,0.010%
    "DNA linking",			1847,0.888%
    "L-DNA linking",			5,0.002%
    "L-RNA linking",			21,0.010%
    "L-beta-peptide, C-gamma linking",	44,0.021%
    "L-gamma-peptide, C-delta linking",	9,0.004%
    "L-peptide COOH carboxy terminus",	51,0.025%
    "L-peptide NH3 amino terminus",	24,0.012%
    "L-peptide linking",		18676,8.981%
    "L-saccharide",			291,0.140%
    "L-saccharide 1, 4 and 1, 4 linking",	2,0.001%
    "RNA OH 3 prime terminus",		8,0.004%
    "RNA linking",			3538,1.701%
    "non-polymer",			166510,80.074%
    "peptide linking",			296,0.142%
    "peptide-like",			1035,0.498%
    "saccharide",			3914,1.882%
"""
###
import sys,os,re,time,logging
#
from xml.etree import ElementTree
#
import rdkit.Chem
#
from ..util import rest
from ..util import xml_utils
#
#############################################################################
def GetProteinData(base_url, pid):
  url = (base_url+'/describePDB?structureId=%s'%pid)
  logging.debug(url)
  try:
    etree = rest.GetURL(url, parse_xml=True)
  except Exception as e:
    logging.error('%s'%(e))
    return []
  #proteins = etree.findall('/PDBdescription/PDB')
  proteins = etree.findall('./PDB')
  logging.debug("Proteins found: %d"%len(proteins))
  data = [protein.attrib for protein in proteins]
  return data

#############################################################################
def GetProteins(base_url, pids, fout):
  n_out=0; tags=[];
  for pid in pids:
    data = GetProteinData(base_url, pid)
    for p in data:
      if n_out==0:
        tags = sorted(p.keys())
        fout.write('\t'.join(tags)+'\n')
      vals = [(p[tag] if tag in p else '') for tag in tags]
      fout.write('\t'.join(vals)+'\n')
      n_out+=1
  logging.info('queries: %d ; proteins out: %d'%(len(pids), n_out))

#############################################################################
def GetLigandData(base_url, pid):
  url = (base_url+'/ligandInfo?structureId=%s'%pid)
  logging.debug(url)
  try:
    etree = rest.GetURL(url, parse_xml=True)
    #ligands = etree.findall('/structureId/ligandInfo/ligand')
    ligands = etree.findall('./ligandInfo/ligand')
  except Exception as e:
    logging.debug('%s'%(e))
    return []
  data=[];
  for ligand in ligands:
    ligdata = ligand.attrib
    for cnode in ligand.iter():
      ligdata[cnode.tag] = cnode.text
    data.append(ligdata)
  return data

#############################################################################
def GetLigands(base_url, pids, druglike, fout):
  n_all=0; n_out=0; n_rejected=0; tags=[];
  for pid in pids:
    data = GetLigandData(base_url, pid)
    for d in data:
      n_all+=1
      if druglike and not LigandIsDruglike(d):
        n_rejected+=1
        continue
      if n_out==0:
        tags = sorted(d.keys())
        fout.write('\t'.join(['pdbid']+tags)+'\n')
      vals = [(d[tag] if tag in d else '') for tag in tags]
      fout.write('\t'.join([pid]+vals)+'\n')
      n_out+=1
  logging.info('queries: %d ; ligands: %d ; ligands out: %d ; rejected: %d'%(len(pids), n_all, n_out, n_rejected))

#############################################################################
def GetLigands_LID2SDF(base_url, lids, fout):
  """Note that each LID many SDFs, one for each occurance. Note also
  base_url arg ignored in this function. May be hacky and unsupported API
  functionality."""
  logging.info('LIDs: %d'%len(lids))
  n_lid=0; n_sdf=0; n_lid2sdf=0; n_notfound=0; n_err=0;
  for lid in lids:
    n_lid+=1
    url = ('https://www.rcsb.org/pdb/download/downloadLigandFiles.do?ligandIdList=%s'%lid)
    logging.debug(url)
    try:
      rval = rest.GetURL(url, parse_json=False)
    except Exception as e:
      logging.error('%s'%(e))
      continue
    if type(rval) is not str:
      logging.error('ERROR: type(RVAL) NOT STR (%s).'%(url))
      logging.debug('ERROR: type(RVAL) NOT STR (%s), type=%s:\n%s'%(url,type(rval),str(rval)))
    elif re.search(r'^<!DOCTYPE html>.*No files found', rval, re.DOTALL|re.I):
      n_notfound+=1
    elif re.search(r'^<!DOCTYPE html>', rval, re.I):
      n_err+=1
      logging.error('ERROR: RVAL NOT SDF (%s).'%(url))
      logging.debug('ERROR: RVAL NOT SDF (%s):\n%s'%(url,rval))
    else:
      fout.write(rval)
      n_sdf_this = len(re.findall(r'\$\$\$\$[\r\n]', rval))
      if n_sdf_this>0: n_lid2sdf+=1
      n_sdf+=n_sdf_this
    if n_lid%1000==0:
      logging.info('LIDs: %d; LID2SDF: %d; not_found: %d; SDFs out: %d'%(n_lid, n_lid2sdf, n_notfound, n_sdf))
  logging.info('LIDs: %d; LID2SDF: %d; not_found: %d; SDFs out: %d'%(n_lid, n_lid2sdf, n_notfound, n_sdf)) 

#############################################################################
def LigandIsDruglike(lig):
  '''Very simple criteria to exclude monoatomic, polymers, etc.'''
  if not lig: return False
  if type(lig) != types.DictType: return False
  ligtype = lig['type'] if 'type' in lig else None
  mf = lig['formula'] if 'formula' in lig else None
  mwt = float(lig['molecularWeight']) if 'molecularWeight' in lig else None
  smi = lig['smiles'] if 'smiles' in lig else None
  if not smi: return False
  if ligtype != 'non-polymer': return False
  if not mwt or mwt < 100.0 or mwt > 1000.0: return False
  if len(re.sub(r'[^A-Z]', '', smi))<6: return False  #Cheap approx atomcount
  try:
    mol = rdkit.Chem.MolFromSmiles(smi)
    if mol and mol.GetNumAtoms()<8: return False
  except Exception as e:
    logging.error("smi = \"%s\" (%s)"%(smi, str(e)))
    return False
  return True

#############################################################################
def GetUniprotData(base_url, pid):
  '''API functionality maybe discontinued.'''
  url = base_url+'/das/pdb_uniprot_mapping/alignment?query=%s'%pid
  try:
    etree = rest.GetURL(url, parse_xml=True)
  except Exception as e:
    logging.debug('%s'%(e))
  if not etree:
    return []
  #logging.debug('%s'%dom.toprettyxml())
  data=[];
  al={}
  alignments = etree.findall('/dasalignment/alignment/alignObject')
  for alignment in alignments:
    if xml_utils.DOM_GetNodeAttr(alignment, 'dbSource')=='UniProt':
      for i in range(alignment.attributes.length):
        attr = alignment.attributes.item(i)
        al[attr.name]=attr.value
    #for cnode in alignment.childNodes:
    #  if cnode.nodeName =='#text': continue
    #  al[cnode.nodeName]=xml_utils.DOM_NodeText(cnode)
    data.append(al)
  return data

#############################################################################
def GetUniprots(base_url, pids, fout):
  n_out=0; tags=[];
  for pid in pids:
    data=GetUniprotData(base_url, pid)
    for d in data:
      if n_out==0:
        tags=sorted(d.keys())
        fout.write('\t'.join(['queryId']+tags)+'\n')
        logging.debug('tags = %s'%str(tags))
      vals=[(d[tag] if tag in d else '') for tag in tags]
      fout.write('\t'.join([pid]+vals)+'\n')
      fout.flush()
      n_out+=1
  logging.info('queries: %d ; uniprots out: %d'%(len(pids), n_out))

#############################################################################
#def AllPIDs(base_url):
#  try:
#    etree = rest.GetURL(base_url+'/getCurrent', parse_xml=True)
#  except Exception as e:
#    logging.error('%s'%(e))
#    return []
#  #proteins = etree.findall('/current/PDB')
#  proteins = etree.findall('./PDB')
#  pids = [protein.get('structureId') for protein in proteins]
#  return pids
#
#############################################################################
def AllPIDs(base_url):
  try:
    rval = rest.GetURL(base_url+'/getCurrent', parse_json=True)
  except Exception as e:
    logging.error('HTTP Error: %s'%(e))
    return []
  pids = rval['idList']
  return pids

#############################################################################
def ShowCounts(base_url):
  pids = AllPIDs(base_url)
  logging.info("IDs: %d"%(len(pids)))

#############################################################################
def ListProteins(base_url, fout):
  pids = AllPIDs(base_url)
  logging.info("IDs: %d"%(len(pids)))
  GetProteins(base_url, pids, fout)

#############################################################################
def ListLigands(base_url, druglike, fout):
  pids = AllPIDs(base_url)
  logging.info("IDs: %d"%(len(pids)))
  GetLigands(base_url, pids, druglike, fout)

#############################################################################
def SearchByKeywords(base_url, kwds):
  query_xml='''\
<?xml version="1.0" encoding="UTF-8"?>
<orgPdbQuery>
<queryType>org.pdb.query.simple.AdvancedKeywordQuery</queryType>
<description>Text Search for: "%(KWDS)s"</description>
<keywords>%(KWDS)s</keywords>
</orgPdbQuery>
'''%{'KWDS':(','.join(kwds))}
  try:
    txt = rest.PostURL(base_url+'/search', data=query_xml, parse_xml=False)
  except Exception as e:
    logging.error('%s'%(e))
    return []
  pids = [pid.strip() for pid in txt.splitlines()]
  return pids

#############################################################################
def SearchByUniprot(base_url, uniprots, fout):
  n_out_total=0; n_pids_total=0;
  tags=["structureId", "title", "expMethod", "keywords", "pubmedId", "resolution", "status"]
  fout.write('\t'.join(['uniprot']+tags)+'\n')

  for uniprot in uniprots:
    query_xml='''\
<?xml version="1.0" encoding="UTF-8"?>
<orgPdbQuery>
<queryType>org.pdb.query.simple.UpAccessionIdQuery</queryType>
<description>Simple query for a list of Uniprot Accession ID: "%(UNIPROT)s"</description>
<accessionIdList>%(UNIPROT)s</accessionIdList>
</orgPdbQuery>
'''%{'UNIPROT':uniprot}
    try:
      txt=rest.PostURL(base_url+'/search', data=query_xml, parse_xml=False)
    except Exception as e:
      logging.error('%s'%(e))
    pids=[]
    for pid in txt.splitlines():
      if pid: pids.append(pid)

    n_out=0;
    #tags=[];
    for pid in pids:
      data = GetProteinData(base_url, pid)
      for p in data:
        #if n_out==0:
        #  tags=sorted(p.keys())
        #  fout.write('\t'.join(['uniprot']+tags)+'\n')
        vals=[(p[tag] if tag in p else '') for tag in tags]
        fout.write('\t'.join([uniprot]+vals)+'\n')
        n_out+=1
    logging.info('query: %s ; pdbids: %d ; out: %d'%(uniprot, len(pids), n_out))
    n_out_total+=n_out;
    n_pids_total+=len(pids)
  logging.info('Total queries: %d ; pdbids: %d ; out: %d'%(len(uniprots), n_pids_total, n_out_total))

#############################################################################



#!/usr/bin/env python3
#############################################################################
'''
Functions for BRENDA API (SOAP) access.

"EC" refers to Enzyme Commission.  EC Numbers are assigned based on functionality,
the catalyzed reaction, not the protein itself.  Hence multiple proteins can 
exist for a given ECN or ECN+organism.

Default is human enzymes, organism="Homo sapiens".

Ref: http://www.brenda-enzymes.org/soap.php

Ref:	BRENDA: a resource for enzyme data and metabolic information.
	Schomburg I, Chang A, Hofmann O, Ebeling C, Ehrentreich F, Schomburg D.
	Trends Biochem Sci. 2002 Jan;27(1):54-6.
	http://www.ncbi.nlm.nih.gov/pubmed/11796225

Ref:	BRENDA in 2015: exciting developments in its 25th year of existence.
	Nucleic Acids Res. 2014 Nov 5. pii: gku1068. [Epub ahead of print]
	Chang A, Schomburg I, Placzek S, Jeske L, Ulbrich M, Xiao M, Sensen CW, Schomburg D.
	http://www.ncbi.nlm.nih.gov/pubmed/25378310.

Ref: http://en.wikipedia.org/wiki/Enzyme_inhibitor
'''
#############################################################################
import sys,os,time,string,re,json,logging,zeep,yaml

#############################################################################
def ReadParamFile(fparam):
  params={};
  with open(fparam, 'r') as fh:
    for param in yaml.load_all(fh, Loader=yaml.BaseLoader):
      for k,v in param.items():
        params[k] = v
  return params

#############################################################################
def SoapAPIClient(wsdl_url):
  client = zeep.Client(wsdl_url)
  return client

#############################################################################
def Test(client, api_params):
  params = tuple(api_params+["ecNumber*1.1.1.1", "organism*Homo sapiens", "kmValue*", "kmValueMaximum*", "substrate*", "commentary*", "ligandStructureId*", "literature*"])
  resultString = client.service.getKmValue(*params)
  print(resultString)

#############################################################################
def GetECN(client, api_params, ecns, fout):
  for ecn in ecns:
    params = tuple(api_params+["ecNumber*%s"%ecn])
    #results = client.service.getEcNumber(*params)
    results = client.service.getEcNumbersFromEcNumber(*params)
    logging.debug("type(results) = %s"%type(results))
    print(str(results))

#############################################################################
def ListECNumbers(client, api_params, fout):
  '''We think this gets all EC numbers.  But maybe not. 6549 on Jan 7 2015.'''
  rstr = client.getEcNumbersFromEcNumber(api_params)
  results = brenda.ParseResultListString(rstr)
  OutputResultsList(results, fout)

#############################################################################
### List all enzymes linked to synonyms.  (5000+ hits but not all.)
def ListECNumbersFromSynonyms(client, api_params, fout):
  rstr = client.getEcNumbersFromSynonyms(api_params)
  results = brenda.ParseResultListString(rstr)
  OutputResultsList(results, fout)

#############################################################################
### List all enzymes linked to inhibitors.
def ListECNumbersFromInhibitors(client, api_params, fout):
  rstr = client.getEcNumbersFromInhibitors(api_params)
  results = brenda.ParseResultListString(rstr)
  OutputResultsList(results, fout)

#############################################################################
### List all enzymes linked to activators.
def ListECNumbersFromActivators(client, api_params, fout):
  rstr = client.getEcNumbersFromActivatingCompound(api_params)
  results = brenda.ParseResultListString(rstr)
  OutputResultsList(results, fout)

#############################################################################
### List all enzymes linked to Ki value.
def ListECNumbersFromKiValue(client, api_params, fout):
  rstr = client.getEcNumbersFromKiValue(api_params)
  results = brenda.ParseResultListString(rstr)
  OutputResultsList(results, fout)

#############################################################################
### List all enzymes linked to Km value.
def ListECNumbersFromKmValue(client, api_params, fout):
  rstr = client.getEcNumbersFromKmValue(api_params)
  results = brenda.ParseResultListString(rstr)
  OutputResultsList(results, fout)

#############################################################################
def ListOrganisms(client, api_params, fout):
  rstr = client.getOrganismsFromOrganism(api_params)
  results = brenda.ParseResultListString(rstr)
  OutputResultsList(results, fout)

#############################################################################
def GetInhibitorData(client, api_params, ecns, organism, fout):
  '''Get data for input ECNs, CSV output.  Focus on inhibitors and Kis.
Per Sandra Placzek, the "literature" field same as "reference" ID.
'''
  tags = ['ecNumber','systematicName','organism']
  ki_tags = ["commentary", "inhibitor", "kiValue", "kiValueMaximum", "ligandStructureId", "literature"]
  fout.write((','.join(tags+ki_tags))+'\n')
  n_in=0; n_out=0;
  for ecn in ecns:
    n_in+=1
    row={tag:'' for tag in tags}
    row['ecNumber'] = ecn
    row['organism'] = organism
    try:
      rstr = GetSystematicName(client, api_params, ecn)
      #logging.debug('rstr="%s"'%rstr)
      results = ParseResultDictString(rstr)
      val = results[0]['systematicName'] if (results and results[0].has_key('systematicName')) else ''
      row['systematicName'] = val
    except Exception as e:
      logging.error('%s'%str(e))

    ki_rows=[]
    try:
      rstr = GetKiValues(client, api_params, ecn, organism)
      results = ParseResultDictString(rstr)
      for result in results:
        ki_row = {}
        for tag in ki_tags:
          val = result[tag] if result.has_key(tag) else ''
          ki_row[tag] = val
        ki_rows.append(ki_row)
    except Exception as e:
      logging.error('%s'%str(e))

    ###Ignore Ki-less records.###if not ki_rows: ki_rows=[{tag:'' for tag in ki_tags}]
    for ki_row in ki_rows:
      for j,tag in enumerate(tags):
        fout.write('%s"%s"'%((',' if j>0 else ''), row[tag]))
      for tag in ki_tags:
        fout.write(',"%s"'%(ki_row[tag]))
      fout.write('\n')
      n_out+=1
  logging.info('input ECNs: %d ; output rows: %d'%(n_in,n_out))

#############################################################################
def GetSequenceData(client, api_params, ecns, organism, fout):
  '''Get data for input ECNs, CSV output.  Focus on sequences/proteins.
Note "firstAccessionCode" is UniProt ID.  What is "id"?
'''
  tags = ['ecNumber','systematicName','organism']
  seq_tags = ["firstAccessionCode","id","noOfAminoAcids","sequence","source"]
  fout.write((','.join(tags+seq_tags))+'\n')
  n_in=0; n_out=0;
  for ecn in ecns:
    n_in+=1
    row = {tag:'' for tag in tags}
    row['ecNumber'] = ecn
    row['organism'] = organism
    try:
      rstr = GetSystematicName(client, api_params, ecn)
      #logging.debug('rstr="%s"'%rstr)
      results = ParseResultDictString(rstr)
      val = results[0]['systematicName'] if (results and results[0].has_key('systematicName')) else ''
      row['systematicName'] = val
    except Exception as e:
      logging.error('%s'%str(e))

    seq_rows=[]
    try:
      rstr = GetSequence(client, api_params, ecn, organism)
      results = ParseResultDictString(rstr)
      for result in results:
        seq_row={}
        for tag in seq_tags:
          val = result[tag] if result.has_key(tag) else ''
          seq_row[tag] = val
        seq_rows.append(seq_row)
    except Exception as e:
      logging.error('%s'%str(e))

    ###Ignore sequence-less records.###if not seq_rows: seq_rows=[{tag:'' for tag in seq_tags}]
    for seq_row in seq_rows:
      for j,tag in enumerate(tags):
        fout.write('%s"%s"'%((',' if j>0 else ''),row[tag]))
      for tag in seq_tags:
        fout.write(',"%s"'%(seq_row[tag]))
      fout.write('\n')
      n_out+=1
  logging.info('input ECNs: %d ; output rows: %d'%(n_in,n_out))

#############################################################################
def GetLigandData(client, api_params, ecns, organism, fout):
  '''Get data for input ECNs, CSV output.  Focus on ligands, which includes substrates,
inhibitors and activators.
'''
  tags = ['ecNumber','systematicName','organism']
  lig_tags = ["ligand","ligandStructureId","role"]
  fout.write((','.join(tags+lig_tags))+'\n')
  n_in=0; n_out=0;
  for ecn in ecns:
    n_in+=1
    row = {tag:'' for tag in tags}
    row['ecNumber'] = ecn
    row['organism'] = organism
    try:
      rstr = GetSystematicName(client, api_params, ecn)
      #logging.debug('rstr="%s"'%rstr)
      results = ParseResultDictString(rstr)
      val = results[0]['systematicName'] if (results and results[0].has_key('systematicName')) else ''
      row['systematicName'] = val
    except Exception as e:
      logging.error('%s'%str(e))

    lig_rows=[]
    try:
      rstr = GetLigands(client, api_params, ecn, organism)
      results = ParseResultDictString(rstr)
      for result in results:
        lig_row={}
        for tag in lig_tags:
          val = result[tag] if result.has_key(tag) else ''
          lig_row[tag] = val
        lig_rows.append(lig_row)
    except Exception as e:
      logging.error('%s'%str(e))

    ###Ignore ligand-less records.###if not lig_rows: lig_rows=[{tag:'' for tag in lig_tags}]
    for lig_row in lig_rows:
      for j,tag in enumerate(tags):
        fout.write('%s"%s"'%((',' if j>0 else ''), row[tag]))
      for tag in lig_tags:
        fout.write(',"%s"'%(lig_row[tag]))
      fout.write('\n')
      n_out+=1
  logging.info('input ECNs: %d ; output rows: %d'%(n_in, n_out))

#############################################################################
def GetReferenceData(client, api_params, ecns, organism, fout):
  '''Get data for input ECNs, CSV output.  Focus on references.'''
  tags = ['ecNumber','systematicName','organism']
  ref_tags = ['ecNumber','journal','authors','pubmedId','reference','title','volume','year','pages','commentary']
  fout.write((','.join(tags+ref_tags))+'\n')
  n_in=0; n_out=0;
  for ecn in ecns:
    n_in+=1
    row = {tag:'' for tag in tags}
    row['ecNumber'] = ecn
    row['organism'] = organism
    try:
      rstr = GetSystematicName(client, api_params, ecn)
      #logging.debug('rstr="%s"'%rstr)
      results = ParseResultDictString(rstr)
      val = results[0]['systematicName'] if (results and results[0].has_key('systematicName')) else ''
      row['systematicName'] = val
    except Exception as e:
      logging.error('%s'%str(e))

    ref_rows=[]
    try:
      rstr = GetReferences(client, api_params, ecn, organism)
      results = ParseResultDictString(rstr)
      for result in results:
        ref_row = {}
        for tag in ref_tags:
          val = result[tag] if result.has_key(tag) else ''
          ref_row[tag] = val
        ref_rows.append(ref_row)
    except Exception as e:
      logging.error('%s'%str(e))

    ###Ignore reference-less records.###if not ref_rows: ref_rows=[{tag:'' for tag in ref_tags}]
    for ref_row in ref_rows:
      for j,tag in enumerate(tags):
        fout.write('%s"%s"'%((',' if j>0 else ''), row[tag]))
      for tag in ref_tags:
        fout.write(',"%s"'%(ref_row[tag]))
      fout.write('\n')
      n_out+=1
  logging.info('input ECNs: %d ; output rows: %d'%(n_in, n_out))

#############################################################################
def QueryString(ecn, organism):
  qstr = ('ecNumber*%s%s'%(ecn, ('#organism*%s'%organism) if organism else ''))
  logging.debug(qstr)
  return qstr

#############################################################################
def GetNames(client, api_params, ecns, organism, fout):
  for ecn in ecns:
    rstr = client.getEnzymeNames(api_params+','+QueryString(ecn, organism))
    results = ParseResultDictString(rstr)
    OutputResultsDict(results, fout)

#############################################################################
def GetSystematicName(client, api_params, ecns, fout):
  '''Organism param not allowed.'''
  for ecn in ecns:
    qstr = ('ecNumber*%s'%(ecn))
    logging.debug(qstr)
    rstr = client.getSystematicName(api_params+','+qstr)
    results = ParseResultDictString(rstr)
    OutputResultsDict(results, fout)

#############################################################################
def GetOrganism(client, api_params, ecns, organism, fout):
  for ecn in ecns:
    rstr = client.getOrganism(api_params+','+QueryString(ecn, organism))
    results = ParseResultDictString(rstr)
    OutputResultsDict(results, fout)

#############################################################################
def GetSequence(client, api_params, ecns, organism, fout):
  '''Supposedly also accepts uniprot param.  How?'''
  for ecn in ecns:
    rstr = client.getSequence(api_params+','+QueryString(ecn, organism))
    results = ParseResultDictString(rstr)
    OutputResultsDict(results, fout)

#############################################################################
def GetPdb(client, api_params, ecns, organism, fout):
  for ecn in ecns:
    rstr = client.getPdb(api_params+','+QueryString(ecn, organism))
    results = ParseResultDictString(rstr)
    OutputResultsDict(results, fout)

#############################################################################
def GetKiValues(client, api_params, ecns, organism, fout):
  for ecn in ecns:
    rstr = client.getKiValue(api_params+','+QueryString(ecn, organism))
    results = ParseResultDictString(rstr)
    OutputResultsDict(results, fout)

#############################################################################
def GetKmValues(client, api_params, ecns, organism, fout):
  for ecn in ecns:
    rstr = client.getKmValue(api_params+','+QueryString(ecn, organism))
    results = ParseResultDictString(rstr)
    OutputResultsDict(results, fout)

#############################################################################
def GetInhibitors(client, api_params, ecns, organism, fout):
  for ecn in ecns:
    rstr = client.getInhibitors(api_params+','+QueryString(ecn, organism))
    results = ParseResultDictString(rstr)
    OutputResultsDict(results, fout)

#############################################################################
def GetActivators(client, api_params, ecns, organism, fout):
  for ecn in ecns:
    rstr = client.getActivatingCompound(api_params+','+QueryString(ecn, organism))
    results = ParseResultDictString(rstr)
    OutputResultsDict(results, fout)

#############################################################################
def GetLigands(client, api_params, ecns, organism, fout):
  for ecn in ecns:
    rstr = client.getLigands(api_params+','+QueryString(ecn, organism))
    results = ParseResultDictString(rstr)
    OutputResultsDict(results, fout)

#############################################################################
def GetReferences(client, api_params, ecns, organism, fout):
  for ecn in ecns:
    qstr = ('ecNumber*%s%s'%(ecn, ('#organism*%s'%organism) if organism else ''))
    qstr+=('#textmining*0') #manually-annotated entries only
    logging.debug(qstr)
    rstr = client.getReference(api_params+','+qstr)
    results = ParseResultDictString(rstr)
    OutputResultsDict(results, fout)

#############################################################################
def ParseResultDictString(rstr):
  results=[]
  if not rstr.strip(): return results
  ecns = re.split(r'!', rstr.strip())
  for ecn in ecns:
    result = {}
    lines = re.split(r'\#', ecn)
    for line in lines:
      if not line.strip(): continue
      #logging.debug('line = "%s"'%line)
      key = re.sub(r'\*.*$','', line)
      val = re.sub(r'^[^\*]*\*','', line)
      result[key]=val
    results.append(result)
  return results

#############################################################################
def ParseResultListString(rstr):
  results =  re.split(r'!', rstr)
  return results

#############################################################################
#def OutputResultsDict(results, fout):
#  n_out=0;
#  for result in results:
#    n_out+=1
#    for key in sorted(result.keys()):
#      fout.write('\t%28s: %s\n'%(key, result[key]))
#  logging.info('results: %d'%n_out)
#
#############################################################################
def OutputResultsDict(results, fout):
  '''Output JSON list of dicts.'''
  fout.write(json.dumps(results, indent=2, sort_keys=True))
  logging.info('results: %d'%len(results))

#############################################################################
def OutputResultsList(results, fout):
  '''Output simple list of values.'''
  n_out=0;
  for result in results:
    n_out+=1
    fout.write('%s\n'%(result))
  logging.info('results: %d'%n_out)

#############################################################################



#!/usr/bin/env python
#############################################################################
### pubchem_gini_index.py - for specified SIDs and AIDs, generate Gini Index
### and relevant data for each compound.
###
### Each assay is an individual.  For each assay, Gini default "wealth" 
### function = 
###     1.0 if active 
###     0.0 if inactive 
###     0.5 if inconclusive 
###     avg if multiple
###
#############################################################################
### activity outcomes: 
###   1 = inactive
###   2 = active
###   3 = inconclusive
###   4 = unspecified
###   multiple, differing 1, 2 or 3 = discrepant
###   not 4 = tested
#############################################################################
### Jeremy Yang
###  28 Apr 2009
#############################################################################
import os,sys,re,getopt,gzip,tempfile
try:
  import gdbm
except:
  import dbm as gdbm

from ... import pubchem
from ...util import gini_utils

PROG=os.path.basename(sys.argv[0])
DATADIR="/pangolin_home/data/pubchem/bioassay/csv/data"

#############################################################################
if __name__=='__main__':
  usage='''
  %(PROG)s - Gini selectivity analysis from assay CSV files

  options:
  --indir=<DIR>   ... dir w/ csv.gz assay data [%(DATADIR)s]
  or
  --infile=<FILE>   ... input csv.gz assay data

  --aids=<AIDS> ... list of AIDs to select within --indir dir
  --aidfile=<AIDFILE> ... file of AIDs to select within --indir dir

  --sids=<SIDS> ... list of SIDs to select
  --sidfile=<SIDFILE> ... file of SIDs to select (ignore others)

  --out_gini=<outfile> ... SIDs w/ activity counts and Gini
  --out_raw=<outfile> ... SIDs w/ scores per assay

  --use_cids     ... use CIDs instead of SIDs (all i/o)
  --resume_gdbm_file=<gdbm_file> ... resume job with existing gdbm file
  --v            ... verbose
  --vv           ... very verbose
  --h            ... this help
'''%{'PROG':PROG,'DATADIR':DATADIR}

  def ErrorExit(msg):
    print >>sys.stderr,msg
    sys.exit(1)

  indir=DATADIR;
  infile=None; verbose=0;
  out_gini_file=None;
  out_raw_file=None;
  aids=None; aidfile=None; sids=None; sidfile=None;
  use_cids=False; resume_gdbm_file=None;
  opts,pargs = getopt.getopt(sys.argv[1:],'',['h','v','vv','indir=',
  'aids=','aidfile=','infile=','out_gini=','resume_gdbm_file=',
  'out_raw=',
  'sids=','sidfile=','use_cids'])
  if not opts: ErrorExit(usage)
  for (opt,val) in opts:
    if opt=='--h': ErrorExit(usage)
    elif opt=='--indir': indir=val
    elif opt=='--infile': infile=val
    elif opt=='--out_gini': out_gini_file=val
    elif opt=='--out_raw': out_raw_file=val
    elif opt=='--aids': aids=val
    elif opt=='--aidfile': aidfile=val
    elif opt=='--sids': sids=val
    elif opt=='--sidfile': sidfile=val
    elif opt=='--use_cids': use_cids=True
    elif opt=='--resume_gdbm_file': resume_gdbm_file=val
    elif opt=='--vv': verbose=2
    elif opt=='--v': verbose=1
    else: ErrorExit('Illegal option: %s'%val)

  idtag='SID'
  if use_cids: idtag='CID'

  if not (indir or infile):
    ErrorExit('input dir or file required\n'+usage)

  aidset=[]
  if aids:
    for val in re.split('[\s,]+',aids):
      if not val.strip(): continue
      aidset.append(int(val.strip()))
  elif aidfile:
    f=open(aidfile)
    if not f:
      ErrorExit('cannot open: %s'%aidfile)
    lines=f.readlines()
    for line in lines:
      if not line.strip(): continue
      aidset.append(int(line.strip()))

  sidset={}	## use a hash for speed (could be many sids)
  if sids:
    for val in re.split('[\s,]+',sids):
      if not val.strip(): continue
      sidset[int(val.strip())]=True
  elif sidfile:
    f=open(sidfile)
    if not f:
      ErrorExit('cannot open: %s'%sidfile)
    lines=f.readlines()
    for line in lines:
      if not line.strip(): continue
      line=re.sub('\s.*$','',line)
      sidset[int(line.strip())]=True
  if len(sidset.keys())==0: sidset=None
  else:
    print >>sys.stderr, '%s: total %ss selected: %d'%(PROG,idtag,len(sidset.keys()))

  files=[]
  if infile:
    try:
      aid=int(re.sub('\.csv\.gz','',os.path.basename(infile)))
    except:
      ErrorExit('cannot parse AID: "%s"'%infile)
    files.append((infile,aid))
  else:
    aidset_foundfile=[]
    for file in os.listdir(indir):
      if not re.search('\.csv\.gz',file): continue
      try:
        aid=int(re.sub('\.csv\.gz','',file))
      except:
        print >>sys.stderr, 'cannot parse AID: "%s"'%file
        continue
      if aidset and (aid not in aidset): continue
      fpath=indir+'/'+file
      files.append((fpath,aid))
      aidset_foundfile.append(aid)
    if aidset:
      for aid in aidset:
        if aid not in aidset_foundfile:
          print >>sys.stderr, 'ERROR: no file for AID: "%d"'%aid

  fout_gini=None
  if out_gini_file:
    fout_gini=open(out_gini_file,'w')
    if not fout_gini:
      ErrorExit('cannot open: %s'%out_gini_file)
  fout_raw=None
  if out_raw_file:
    fout_raw=open(out_raw_file,'w')
    if not fout_raw:
      ErrorExit('cannot open: %s'%out_raw_file)

  if resume_gdbm_file:
    sids_db_path=resume_gdbm_file
    sids_db=gdbm.open(sids_db_path,'w')
  else:
    tempfile.tempdir=os.environ['HOME']+'/Downloads'
    fd,sids_db_path = tempfile.mkstemp('_sids_db',PROG)
    os.close(fd)
    sids_db=gdbm.open(sids_db_path,'n')
  sids_list=[]

  if fout_gini:
    fout_gini.write('SID\tn_inactive\tn_active\tn_inconclusive\tn_unspecified\tn_discrepant\tn_tested\tgini\n')

  sids_active=[]; sids_inactive=[]; sids_inconclusive=[]; sids_unspecified=[];
  sids_discrepant=[]; sids_tested=[];
  n_datapoints_total=0
  i_file=0
  for fpath,aid in files:
    i_file+=1
    print >>sys.stderr, '%d. [%d]: %s'%(i_file,aid,fpath)
    if sids_db.has_key('AID_DONE_FLAG_%d'%aid):
      continue
    f=gzip.open(fpath)
    ftxt=f.read()
    sids_this=pubchem.ftp.Utils.ExtractOutcomes(ftxt,sidset,use_cids)
    n_active=0; n_inactive=0; n_inconclusive=0; n_unspecified=0; n_discrepant=0;
    for sid in sids_this.keys():

      if not sids_db.has_key('%d'%sid):
        sids_db['%d'%sid]=''
        sids_list.append(sid)

      outcome=sids_this[sid]['outcome']

      sids_db['%d_%d'%(sid,aid)]=('%d'%outcome)
      aids_this=pubchem.ftp.Utils.Str2Ints(sids_db['%d'%(sid)])
      if aid not in aids_this:
        aids_this.append(aid)
      sids_db['%d'%(sid)]=pubchem.ftp.Utils.Ints2Str(aids_this)

      n_datapoints_total+=1
      if outcome==2: n_active+=1
      elif outcome==1: n_inactive+=1
      elif outcome==3: n_inconclusive+=1
      elif outcome==4: n_unspecified+=1
      elif outcome==5: n_discrepant+=1
      else: print >>sys.stderr, 'ERROR: outcome=%d'%(sids_this[sid]['outcome'])

    f.close()
    n_total=n_active+n_inactive+n_inconclusive+n_unspecified+n_discrepant
    n_tested=n_total-n_unspecified
    if verbose>1:
      print >>sys.stderr, '\t       active: %3d'%(n_active)
      print >>sys.stderr, '\t     inactive: %3d'%(n_inactive)
      print >>sys.stderr, '\t inconclusive: %3d'%(n_inconclusive)
      print >>sys.stderr, '\t   discrepant: %3d'%(n_discrepant)
      print >>sys.stderr, '\t(total tested: %3d)'%(n_tested)
      print >>sys.stderr, '\t  unspecified: %3d'%(n_unspecified)
      print >>sys.stderr, '\t        total: %3d'%(n_total)

    sids_db['AID_DONE_FLAG_%d'%aid]='done'	#file done; for resume


  n_ginis=0;
  for sid in sids_list:
    if fout_raw:
      fout_raw.write('%d'%(sid))
    aids_this=pubchem.ftp.Utils.Str2Ints(sids_db['%d'%(sid)])
    if not aids_this:
      pass	#?
    scores=[]
    n_total=0; n_active=0; n_inactive=0; n_inconclusive=0;
    n_unspecified=0; n_discrepant=0;
    for aid in aids_this:
      outcome=sids_db['%d_%d'%(sid,aid)]
      if outcome=='1':
        score=0.0 # 1:inactive
        n_inactive+=1
      elif outcome=='2':
        score=1.0 # 2:active
        n_active+=1
      elif outcome=='3':
        score=0.5 # 3:inconclusive
        n_inconclusive+=1
      elif outcome=='4':
        n_unspecified+=1
      elif outcome=='5':
        score=0.5 # 5:discrepant
        n_discrepant+=1
      else: continue	#error
      scores.append(score)
      if fout_raw:
        fout_raw.write(',%d:%.1f'%(aid,score))
    if fout_raw:
      fout_raw.write('\n')

    n_total=n_active+n_inactive+n_inconclusive+n_unspecified+n_discrepant
    n_tested=n_total-n_unspecified

    if not scores:
      pass	#?

    gini=gini_utils.GiniProcessor(scores)
    n_ginis+=1
    if fout_gini:
      fout_gini.write('%d\t%d\t%d\t%d\t%d\t%d\t%d\t%.2f\n'%(sid,n_inactive,n_active,n_inconclusive,n_unspecified,n_discrepant,n_tested,gini))


  if fout_gini: fout_gini.close()
  if fout_raw: fout_raw.close()

  os.unlink(sids_db_path)
  print >>sys.stderr, 'DEBUG: gdbm file: %s'%(sids_db_path)

  print >>sys.stderr, '%s: number of assay files: %d'%(PROG,len(files))
  print >>sys.stderr, '%s: total %ss: %d'%(PROG,idtag,len(sids_list))
  print >>sys.stderr, '%s: total Gini Indices: %d'%(PROG,n_ginis)
  print >>sys.stderr, '%s: total datapoints: %d'%(PROG,n_datapoints_total)




#!/usr/bin/env python3
###
import sys,os,json,logging

import gql
from gql.transport.requests import RequestsHTTPTransport

#############################################################################
def RunQuery(gql_url, graphql, fout):
  if graphql is None:
    logging.error("No query.")
    return

  _transport = RequestsHTTPTransport(gql_url)
  client = gql.Client(transport=_transport, fetch_schema_from_transport=False)
  logging.debug(f"client.schema: '{client.schema}'")

  try:
    query = gql.gql(graphql)
    rval = client.execute(query)
    fout.write(json.dumps(rval, indent=2)+"\n")
  except Exception as e:
    logging.error(e)

#############################################################################





#!/usr/bin/env python3
"""
LINCS REST API client
New (2019) iLINCS: 
http://www.ilincs.org/ilincs/APIinfo
http://www.ilincs.org/ilincs/APIdocumentation
(http://lincsportal.ccs.miami.edu/dcic/api/ DEPRECATED?)
"""
###
import sys,os,argparse,re,time,json,logging
#
from .. import lincs
#
#############################################################################
if __name__=='__main__':
  epilog="""\
Examples:
NCBI Gene IDs: 207;
PerturbagenIDs: BRD-A00100033 (get_compound);
LINCS PertIDs: LSM-2121;
Perturbagen-Compound IDs: LSM-2421;
Signature IDs: LINCSCP_10260,LINCSCP_10261,LINCSCP_10262;
Dataset IDs: EDS-1013,EDS-1014;
Search Terms: cancer, vorinostat, MCF7.
"""
  parser = argparse.ArgumentParser(description=f'LINCS REST API client ({lincs.API_HOST})', epilog=epilog)
  ops = [
	'get_gene', 'get_compound', 'get_dataset',
	'list_genes', 'list_compounds', 'list_datasets',
	'search_dataset', 'search_signature',
	'get_signature'
	]
  parser.add_argument("op", choices=ops, help='OPERATION')
  parser.add_argument("--i", dest="ifile", help="input file, IDs")
  parser.add_argument("--o", dest="ofile", help="output (TSV)")
  parser.add_argument("--ids", help="input IDs, comma-separated")
  parser.add_argument("--searchTerm", dest="searchTerm", help="Entity searchTerm e.g. Rock1)")
  parser.add_argument("--lincs_only", action="store_true", help="LINCS datasets only")
  parser.add_argument("--ngene", type=int, default=50, help="top genes per signature")
  parser.add_argument("--nmax", type=int, help="max results")
  parser.add_argument("--skip", type=int, help="skip results")
  parser.add_argument("--api_host", default=lincs.API_HOST)
  parser.add_argument("--api_base_path", default=lincs.API_BASE_PATH)
  parser.add_argument("-v", "--verbose", action="count", default=0)
  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  base_url = 'https://'+args.api_host+args.api_base_path

  fout = open(args.ofile, "w+") if args.ofile else sys.stdout

  ids=[];
  if args.ifile:
    fin = open(args.ifile)
    while True:
      line = fin.readline()
      if not line: break
      ids.append(line.strip())
  elif args.ids:
    ids = re.split('[,\s]+', args.ids.strip())

  if args.op == 'get_gene':
    lincs.GetGene(ids, base_url, fout)

  elif args.op == 'get_compound':
    lincs.GetCompound(ids, base_url, fout)

  elif args.op == 'list_compounds':
    lincs.ListCompounds(base_url, fout)

  elif args.op == 'get_dataset':
    lincs.GetDataset(ids, base_url, fout)

  elif args.op == 'search_dataset':
    lincs.SearchDataset(args.searchTerm, args.lincs_only, base_url, fout)

  elif args.op == 'search_signature':
    lincs.SearchSignature(ids, args.lincs_only, base_url, fout)

  elif args.op == 'get_signature':
    lincs.GetSignature(ids, args.ngene, base_url, fout)

  else:
    parser.error(f'Invalid operation: {args.op}')




#!/usr/bin/env python3
"""
GeneOntology client.
"""
###
import os,sys,argparse,re,json,time,logging
import urllib.parse

from .. import geneontology
#
##############################################################################
if __name__=='__main__':
  epilog="""\
"""
  parser = argparse.ArgumentParser(description='GeneOntolgy API client', epilog=epilog)
  ops=['list_terms', 'list_genes', 'get_entities', 'get_geneTerms' ]
  parser.add_argument("op", choices=ops, help='OPERATION')
  parser.add_argument("--i", dest="ifile", help="input file of IDs")
  parser.add_argument("--ids", help="ID list (comma-separated)(e.g. NCBIGene:84570, GO:0006954)")
  parser.add_argument("--o", dest="ofile", help="output file (TSV)")
  parser.add_argument("--api_host", default=geneontology.API_HOST)
  parser.add_argument("--api_base_path", default=geneontology.API_BASE_PATH)
  parser.add_argument("-v", "--verbose", default=0, action="count")
  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  BASE_URL = 'https://'+args.api_host+args.api_base_path

  fout = open(args.ofile, "w+") if args.ofile else sys.stdout

  if args.ids:
    ids = re.split(r'[,\s]+', args.ids.strip())
  elif args.ifile:
    with open(args.ifile) as fin:
      while True:
        line = fin.readline()
        if not line: break
        ids.append(line.rstrip())

  if args.op == 'list_terms':
    geneontology.ListTerms(BASE_URL, fout)

  elif args.op == 'list_genes':
    geneontology.ListGenes(BASE_URL, fout)

  elif args.op == 'get_entities':
    geneontology.GetEntities(ids, BASE_URL, fout)

  elif args.op == 'get_geneTerms':
    geneontology.GetGeneTerms(ids, BASE_URL, fout)

  else:
    parser.error(f'Invalid operation: {args.op}')




#!/usr/bin/env python3
"""
See: http://www.genenames.org/
"""
###
import sys,os,re,argparse,time,json,logging
import pandas as pd
#
from ..util import rest
from .. import hugo
#
##############################################################################
if __name__=='__main__':
  FTYPES=['SYMBOL', 'ALIAS_SYMBOL', 'HGNC_ID', 'UNIPROT', 'ENTREZ_ID', 'ENSEMBL_GENE_ID', 'VEGA_ID'];
  epilog='''\
           SYMBOL: HGNC gene symbol (e.g. ZNF3),
     ALIAS_SYMBOL: synonym gene symbol (e.g. AAA1),
          HGNC_ID: HGNC gene ID (e.g. HGNC:13089),
          UNIPROT: UniProt ID,
        ENTREZ_ID: NCBI Gene ID,
  ENSEMBL_GENE_ID: Ensembl Gene ID (e.g. ENSG00000003056),
'''
  parser = argparse.ArgumentParser(description='HUGO HGNC REST API client', epilog=epilog)
  ops = ['search', 'get', 'info', 'list_searchable']
  parser.add_argument("op", choices=ops, help='OPERATION')
  parser.add_argument("--o", dest="ofile", help="output (TSV)")
  parser.add_argument("--i", dest="ifile", help="query file")
  parser.add_argument("--query", help="gene query")
  parser.add_argument("--ftypes", default="SYMBOL,ALIAS_SYMBOL", help="comma-separated list")
  parser.add_argument("--ftypes_all", action="store_true", help="(could be slow)")
  parser.add_argument("--rawquery", action="store_true")
  parser.add_argument("--nmax", type=int, default=None, help="max records")
  parser.add_argument("--skip", type=int, default=0, help="skip 1st SKIP queries")
  parser.add_argument("--api_host", default=hugo.API_HOST)
  parser.add_argument("--api_base_path", default=hugo.API_BASE_PATH)
  parser.add_argument("-v", "--verbose", default=0, action="count")
  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  BASE_URL='http://'+args.api_host+args.api_base_path

  fout = open(args.ofile,"w") if args.ofile else sys.stdout

  t0=time.time()

  if args.ifile:
    qrys=[];
    with open(args.ifile) as fin:
      while True:
        line=fin.readline()
        if not line: break
        if line.rstrip(): qrys.append(line.rstrip())
  elif args.query:
    qrys = [args.query]

  if args.ftypes_all:
    ftypes = list(hugo.ListSearchableFields(BASE_URL).iloc[:,0])
  else:
    ftypes = re.split('[, ]+', args.ftypes.strip())
    ftypes = [s.upper() for s in ftypes]
    if len(set(ftypes) - set(FTYPES))>0:
      parser.error("Invalid field type[s]: {0}".format(set(ftypes) - set(FTYPES)))

  if args.op == "info":
    hugo.Info(BASE_URL, fout)

  elif args.op == "list_searchable":
    hugo.ListSearchableFields(BASE_URL, fout)

  elif args.op == "get":
    hugo.GetGenes(qrys, ftypes, args.skip, BASE_URL, fout)

  elif args.op == "search":
    hugo.SearchGenes(qrys, ftypes, BASE_URL, fout)

  else:
    parser.error("Invalid operation: {}".format(args.op))

  logging.info(('elapsed time: %s'%(time.strftime('%Hh:%Mm:%Ss', time.gmtime(time.time()-t0)))))



#!/usr/bin/env python3
"""
Access to Ensembl REST API.
http://rest.ensembl.org/documentation/info/lookup
"""
import sys,os,re,time,json,logging,tqdm
import pandas as pd
import requests
#
API_HOST='rest.ensembl.org'
API_BASE_PATH=''
#
BASE_URL='https://'+API_HOST+API_BASE_PATH
#
##############################################################################
def ShowVersion(base_url=BASE_URL, fout=None):
  df = pd.DataFrame({
      "param": ["EnsEMBL REST API version", "EnsEMBL software API version", "EnsEMBL genomes version"],
      "value": [
          requests.get(base_url+'/info/rest?content-type=application/json').json()['release'],
          requests.get(base_url+'/info/software?content-type=application/json').json()['release'],
          requests.get(base_url+'/info/eg_version?content-type=application/json').json()['version']
          ]})
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_out: {df.shape[0]}")
  return df

##############################################################################
def ListSpecies(base_url=BASE_URL, fout=None):
  tags=None; df=pd.DataFrame();
  rval = requests.get(base_url+'/info/species?content-type=application/json').json()
  specs = rval["species"]
  for spec in specs:
    if not tags: tags = list(spec.keys())
    df = pd.concat([df, pd.DataFrame({tags[j]:[spec[tags[j]]] for j in range(len(tags))})])
  if fout: df.to_csv(fout, "\t", index=False)
  logging.info(f"n_out: {df.shape[0]}")
  return df

##############################################################################
def GetInfo(ids, base_url=BASE_URL, fout=None):
  n_out=0; n_err=0; tags=[]; df=None; tq=None;
  quiet = bool(logging.getLogger().getEffectiveLevel()>15)
  for id_this in ids:
    if tq is not None: tq.update()
    url_this = base_url+'/lookup/id/'+id_this+'?content-type=application/json&expand=0'
    logging.debug(url_this)
    rval = requests.get(url_this, headers={"Content-Type":"application/json"})
    if not rval.ok:
      logging.error(f'{rval.status_code} : "{id_this}"')
      n_err+=1
      continue
    gene = rval.json()
    if not tags:
      for tag in gene.keys():
        if type(gene[tag]) not in (list, dict): tags.append(tag) #Only simple metadata.
    df_this = pd.DataFrame({tags[j]:([str(gene[tags[j]])] if tags[j] in gene else ['']) for j in range(len(tags))})
    if fout is not None:
      df_this.to_csv(fout, "\t", index=False, header=bool(n_out==0))
      n_out += 1
    if fout is None: df = pd.concat([df, df_this])
    if tq is None and not quiet: tq = tqdm.tqdm(total=len(ids), unit="genes")
  if tq is not None: tq.close()
  logging.info(f"n_ids: {len(ids)}; n_out: {n_out}; n_err: {n_err}")
  if fout is None: return df

##############################################################################
def GetXrefs(ids, base_url=BASE_URL, fout=None):
  n_out=0; n_err=0; tags=None; dbcounts={}; df=pd.DataFrame(); tq=None;
  quiet = bool(logging.getLogger().getEffectiveLevel()>15)
  for id_this in ids:
    if tq is not None: tq.update()
    url_this = base_url+'/xrefs/id/'+id_this
    rval = requests.get(url_this, headers={"Content-Type":"application/json"})
    if not rval.ok:
      logging.error(f'{rval.status_code} : "{id_this}"')
      n_err+=1
      continue
    xrefs = rval.json()
    for xref in xrefs:
      if not (type(xref) is dict and 'dbname' in xref): continue
      dbname = xref['dbname']
      if dbname not in dbcounts: dbcounts[dbname]=0
      dbcounts[dbname]+=1
      if not tags: tags = list(xref.keys())
      df_this =  pd.DataFrame({tags[j]:([str(xref[tags[j]])] if tags[j] in xref else ['']) for j in range(len(tags))})
      if fout is None: df = pd.concat([df, df_this])
      else: df_this.to_csv(fout, "\t", index=False)
      n_out+=1
    if tq is None and not quiet: tq = tqdm.tqdm(total=len(ids), unit="genes")
  if tq is not None: tq.close()
  for key in sorted(dbcounts.keys()):
    logging.info(f"Xref counts, db = {key:12s}: {dbcounts[key]:5d}")
  logging.info(f"n_ids: {len(ids)}; n_out: {n_out}; n_err: {n_err}")
  if fout is None: return df

##############################################################################



#!/usr/bin/env python3
##############################################################################
### Medline utilities - access SNOMED and ICD codes
### https://medlineplus.gov/connect/technical.html
### https://medlineplus.gov/connect/service.html
##############################################################################
### https://apps.nlm.nih.gov/medlineplus/services/mpconnect_service.cfm
### Two required parameters:
### 1. Code System (one of): 
###	ICD-10-CM: mainSearchCriteria.v.cs=2.16.840.1.113883.6.90
###	ICD-9-CM: mainSearchCriteria.v.cs=2.16.840.1.113883.6.103
###	SNOMED_CT: mainSearchCriteria.v.cs=2.16.840.1.113883.6.96
###	NDC: mainSearchCriteria.v.cs=2.16.840.1.113883.6.69
###	RXNORM: mainSearchCriteria.v.cs=2.16.840.1.113883.6.88
###	LOINC:	mainSearchCriteria.v.cs=2.16.840.1.113883.6.1
### 2. Code: 
###	mainSearchCriteria.v.c=250.33
###
### Content format:
###	XML (default): knowledgeResponseType=text/xml
###	JSON: knowledgeResponseType=application/json
###	JSONP: knowledgeResponseType=application/javascript&callback=CallbackFunction
###	  where CallbackFunction is a name you give the call back function.
##############################################################################
import sys,os,re,time,logging
import urllib.parse,json
#
from ...util import rest
#
CODESYSTEMS = {
	'SNOWMEDCT':	'2.16.840.1.113883.6.96',
	'ICD9CM':	'2.16.840.1.113883.6.103',
	'ICD10CM':	'2.16.840.1.113883.6.90',
	'NDC'	:	'2.16.840.1.113883.6.69',
	'RXNORM':	'2.16.840.1.113883.6.88',
	'LOINC'	:	'2.16.840.1.113883.6.1'
	}
#
API_HOST='apps.nlm.nih.gov'
API_BASE_PATH='/medlineplus/services/mpconnect_service.cfm'
API_BASE_URL='https://'+API_HOST+API_BASE_PATH
#
##############################################################################
def GetCode(codes, codesys, base_url=API_BASE_URL, fout=None):
  url=base_url
  url+=('?knowledgeResponseType=application/json')
  url+=('&mainSearchCriteria.v.cs='+CODESYSTEMS[codesys])
  for code in codes:
    url_this =url+('&mainSearchCriteria.v.c='+code)
    rval = rest.Utils.GetURL(url_this, parse_json=True)
    logging.debug(json.dumps(rval, sort_keys=True, indent=2))

##############################################################################



#!/usr/bin/env python3
"""
OpenTargets REST API client, using the Python client package.

https://www.targetvalidation.org/documentation/api/
http://opentargets.readthedocs.io/
https://docs.targetvalidation.org/programmatic-access/python-client
https://docs.targetvalidation.org/programmatic-access/rest-api
https://docs.targetvalidation.org/data-sources/genetic-associations
https://platform-api.opentargets.io/v3/platform
https://platform-api.opentargets.io/v3/platform/docs/swagger-ui
"""
import sys,os,re,json,csv,logging

import opentargets

#############################################################################
def SearchAssociations(otclient, ids, idtype, minscore, skip, nmax, fout):
  csvWriter = csv.writer(fout, delimiter='\t', quotechar='"', quoting=csv.QUOTE_MINIMAL)
  tags=None;
  n_id=0; n_assn=0; n_id_found=0;
  for id_this in ids:
    n_id+=1
    if n_id<=skip: continue
    logging.debug('%s:'%id_this)
    try:
      if idtype=="disease":
        assns = otclient.get_associations_for_disease(id_this)
      else:
        assns = otclient.get_associations_for_target(id_this)
    except Exception as e:
      logging.error(str(e))
      continue
    if assns:
      n_id_found+=1
    for assn in assns:
      logging.debug(json.dumps(assn, sort_keys=True, indent=2))
      if not tags:
        tags=["id","is_direct","target_id","gene_name","gene_symbol","disease_id","disease_efo_label","score_overall"]
        tags_assn_score_sources = list(assn['association_score']['datasources'].keys())
        tags_assn_score_types = list(assn['association_score']['datatypes'].keys())
        tags_evidence_count_sources = list(assn['evidence_count']['datasources'].keys())
        tags_evidence_count_types = list(assn['evidence_count']['datatypes'].keys())
        csvWriter.writerow(tags+
		list(map(lambda t:'assn_score_source.%s'%t, tags_assn_score_sources))+
		list(map(lambda t:'assn_score_type.%s'%t, tags_assn_score_types))+
		list(map(lambda t:'evidence_count_source.%s'%t, tags_evidence_count_sources))+
		list(map(lambda t:'evidence_count_type.%s'%t, tags_evidence_count_types)))
      assn_id = assn['id'] if 'id' in assn else ''
      is_direct = assn['is_direct'] if 'is_direct' in assn else ''
      target_id = assn['target']['id'] if 'target' in assn and 'id' in assn['target'] else ''
      gene_name = assn['target']['gene_info']['name'] if 'target' in assn and 'gene_info' in assn['target'] and 'name' in assn['target']['gene_info'] else ''
      gene_symbol = assn['target']['gene_info']['symbol'] if 'target' in assn and 'gene_info' in assn['target'] and 'symbol' in assn['target']['gene_info'] else ''
      disease_id = assn['disease']['id'] if 'disease' in assn and 'id' in assn['disease'] else ''
      disease_efo_label = assn['disease']['efo_info']['label'] if 'disease' in assn and 'efo_info' in assn['disease'] and 'label' in assn['disease']['efo_info'] else ''
      score_overall = assn['association_score']['overall'] if 'association_score' in assn and 'overall' in assn['association_score'] else ''
      vals=[assn_id, is_direct, target_id, gene_name, gene_symbol, disease_id, disease_efo_label, score_overall]
      for tag in tags_assn_score_sources:
        vals.append(assn['association_score']['datasources'][tag] if tag in assn['association_score']['datasources'] else '')
      for tag in tags_assn_score_types:
        vals.append(assn['association_score']['datatypes'][tag] if tag in assn['association_score']['datatypes'] else '')
      for tag in tags_evidence_count_sources:
        vals.append(assn['evidence_count']['datasources'][tag] if tag in assn['evidence_count']['datasources'] else '')
      for tag in tags_evidence_count_types:
        vals.append(assn['evidence_count']['datatypes'][tag] if tag in assn['evidence_count']['datatypes'] else '')

      for i in range(len(vals)):
        if type(vals[i]) is float:
          vals[i] = round(vals[i], 5)
      csvWriter.writerow(vals)
      n_assn+=1
    logging.debug('n_id=%d ; skip=%d ; nmax=%d'%(n_id, skip, nmax))
    if nmax and n_id-skip>=nmax: break
  logging.info('n_id = %d; n_id_found = %d; n_assn = %d'%(n_id-skip, n_id_found, n_assn))

#############################################################################



#!/usr/bin/env python3
"""
Sparql endpoint client utility functions
"""
###
import os,sys,re,json,logging

import SPARQLWrapper

#############################################################################
def SparqlRequest(rq_code, rq_uri, defgraph=None, fmt=None):
  rq_results = None
  logging.debug('URI: %s'%(rq_uri))
  logging.debug('%s'%(rq_code))
  try:
    sparql = SPARQLWrapper.SPARQLWrapper(rq_uri)
    sparql.setQuery(rq_code)
    if fmt:
      sparql.setReturnFormat(fmt)
    rq_results = sparql.query()
  except Exception as e:
    logging.error('Error: %s'%e)
    errtype,val,traceback = sys.exc_info()
    logging.error('sys.exc_info:\n(%s)\n%s'%(errtype,val))
    if traceback: logging.info('traceback:\n%s>'%(traceback))
    logging.error('%s'%str(rq_code))
  return rq_results

#############################################################################
def Results2TSV(results, variables, nmax, fout):
  if variables:
    variables = variables.strip()
    variables = re.split(r'[\s,]+', variables)
  else:
    variables=[]
  if not variables:
    variables = results["head"]["vars"]
  logging.debug("selected vars: %s"%(','.join(variables)))
  if "link" in results["head"]:
    logging.debug("links: %s"%(','.join(results["head"]["link"])))
  if "distinct" in results["results"]:
    logging.debug("distinct: %s"%results["results"]["distinct"])
  if "ordered" in results["results"]:
    logging.debug("ordered: %s"%results["results"]["ordered"])
  n_row=0;
  fout.write('%s\n'%('\t'.join(variables)))
  for binding in results["results"]["bindings"]:
    logging.debug(json.dumps(binding, indent=2))
    row_out=[]
    for var in variables:
      if var in binding:
        val = binding[var]["value"]
        if binding[var]["type"]=='uri':
          val=('<%s>'%val)
      else:
        val=''
      try:
        val=val.encode('utf8')
      except:
        logging.info('unicode error [%d]'%n_row)
        val='error'
      row_out.append(val)
    fout.write('%s\n'%('\t'.join(row_out)))
    n_row+=1
    if n_row==nmax:
      logging.info('NOTE: Output truncated at NMAX = %d.'%nmax)
      break
  logging.info('N = %d'%n_row)

#############################################################################
### Return list of rows, each row list of values.
#############################################################################
def Results2List(results, variables, nmax):
  if variables:
    variables = variables.strip()
    variables = re.split(r'[\s,]+', variables)
  else:
    variables=[]
  if not variables:
    variables = results["head"]["vars"]
  logging.debug("selected vars: %s"%(','.join(variables)))
  if "link" in results["head"]:
    logging.debug("links: %s"%(','.join(results["head"]["link"])))
  if "distinct" in results["results"]:
    logging.debug("distinct: %s"%results["results"]["distinct"])
  if "ordered" in results["results"]:
    logging.debug("ordered: %s"%results["results"]["ordered"])
  n_row=0
  results_out=[]
  for binding in results["results"]["bindings"]:
    logging.debug(json.dumps(binding, indent=2))
    row_out=[]
    for var in variables:
      if var in binding:
        val = binding[var]["value"]
        if binding[var]["type"]=='uri':
          val=('<%s>'%val)
      else:
        val=''
      try:
        val=val.encode('utf8')
      except:
        logging.info('unicode error [%d]'%n_row)
        val='error'
      row_out.append(val)
    results_out.append(row_out)
    n_row+=1
    if n_row==nmax:
      logging.info('NOTE: Output truncated at NMAX = %d.'%nmax)
      break

  logging.info('N = %d'%n_row)

  return results_out

#############################################################################
#  FILTER (lang(?propval) = "en" || lang(?propval) = "" ) ##removes depiction
#  FILTER (lang(?propval) IN ( "en" , "" )) ##removes depiction
def Drugname2Sparql(drugname):
  rq_code='''\
SELECT DISTINCT
  ?drug ?propname ?propval
WHERE
{
  {
    <http://dbpedia.org/resource/%(DRUGNAME)s> rdf:type <http://dbpedia.org/ontology/Drug> .
    <http://dbpedia.org/resource/%(DRUGNAME)s> ?propname ?propval .
  }
  UNION
  {
    ?drug rdf:type <http://dbpedia.org/ontology/Drug> .
    ?drug ?propname ?propval .
    { ?drug <http://dbpedia.org/property/licenceEu> ?synonym . }
    UNION
    { ?drug <http://dbpedia.org/property/licenceUs> ?synonym . }
    UNION
    { ?drug <http://dbpedia.org/property/tradename> ?synonym . }
    UNION
    {
      ?drug rdfs:label ?synonym .
      FILTER (regex(?synonym, "^%(DRUGNAME)s$", "i"))
    }
    FILTER (regex(?synonym, "^%(DRUGNAME)s$", "i"))
  }
}
'''%{'DRUGNAME':drugname}
  return rq_code

#############################################################################
def Test(drugname="metformin", fmt=SPARQLWrapper.JSON):
  RQ_URI='http://dbpedia.org/sparql'
  logging.info('drugname: "%s"; endpoint: %s'%(drugname, RQ_URI))
  rq_code = Drugname2Sparql(drugname)
  logging.debug('sparql:\n%s'%(rq_code))
  rq_results = SparqlRequest(rq_code, RQ_URI, defgraph=None, fmt=fmt)
  if fmt in (SPARQLWrapper.JSON, SPARQLWrapper.JSONLD):
    print(json.dumps(rq_results.convert(), indent=2))
  elif fmt in (SPARQLWrapper.XML, SPARQLWrapper.RDFXML, SPARQLWrapper.RDF):
    print(rq_results.convert().toxml())
  else:
    print(rq_results.convert().decode("utf-8"))

#############################################################################



#!/usr/bin/env python3
"""
Pharos GraphQL API client
https://pharos.nih.gov/api
"""
###
import sys,os,argparse,json,re,time,logging
#
from ... import idg
#
#############################################################################
if __name__=='__main__':
  parser = argparse.ArgumentParser(description='Pharos GraphQL API client')
  ops = [
	'get_targets',
	'get_diseases',
	'test' ]
  parser.add_argument("op", choices=ops, help='OPERATION')
  parser.add_argument("--i", dest="ifile", help="input file, target IDs")
  parser.add_argument("--ids", dest="ids", help="IDs, target, comma-separated")
  parser.add_argument("--o", dest="ofile", help="output (TSV)")
  parser.add_argument("--idtype_target", choices=list(idg.pharos.Utils.IDTYPES_TARGET.keys()), default='sym', help="target ID type")
  parser.add_argument("--idtype_disease", choices=list(idg.pharos.Utils.IDTYPES_DISEASE.keys()), default='name', help="disease ID type")
  parser.add_argument("--nmax", type=int, help="max to return")
  parser.add_argument("--api_endpoint", default=idg.pharos.Utils.API_ENDPOINT)
  parser.add_argument("-v", "--verbose", default=0, action="count")
  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  fout = open(args.ofile, "w+") if args.ofile else sys.stdout

  ids=[];
  if args.ifile:
    fin = open(args.ifile)
    while True:
      line = fin.readline()
      if not line: break
      ids.append(line.rstrip())
    fin.close()
  elif args.ids:
    ids = re.split(r'\s*,\s*', args.ids.strip())
  logging.info(f"Input IDs: {len(ids)}")

  t0 = time.time()

  if re.match(r'^get_', args.op) and not ids:
    parser.error(f"{args.op} requires IDs.")

  if args.op=='get_targets':
    idg.pharos.Utils.GetTargets(ids, args.idtype_target, args.api_endpoint, fout)

  elif args.op=='get_diseases':
    idg.pharos.Utils.GetDiseases(ids, args.idtype_disease, args.api_endpoint, fout)

  elif args.op=='test':
    idg.pharos.Utils.Test(args.api_endpoint, fout)

  else:
    logging.error(f"Invalid operation: {args.op}")

  logging.info("Elapsed time: "+time.strftime('%Hh:%Mm:%Ss', time.gmtime(time.time()-t0)))



#!/usr/bin/env python3
"""
https://query.wikidata.org/sparql

https://www.wikidata.org/wiki/User:ProteinBoxBot/SPARQL_Examples

PREFIX wd: <http://www.wikidata.org/entity/>
PREFIX wdt: <http://www.wikidata.org/prop/direct/>
PREFIX bd: <http://www.bigdata.com/rdf#>
PREFIX up: <http://purl.uniprot.org/core/>
PREFIX uniprotkb:<http://purl.uniprot.org/uniprot/>

GeneWiki:
  * <https://pubmed.ncbi.nlm.nih.gov/26989148/>
"""
###
import sys,os,argparse,logging
import pandas as pd

import wikidataintegrator

from .. import wikidata

#############################################################################
if __name__=="__main__":
  verstr = (f"Python: {sys.version.split()[0]}; wikidataintegrator: {wikidataintegrator.__version__}")
  parser = argparse.ArgumentParser(description="Wikidata utilities with biomedical focus", epilog=verstr)
  ops = ["list_drugTargetPairs", "list_geneDiseasePairs", "test"]
  parser.add_argument("op", choices=ops, help="OPERATION (select one)")
  parser.add_argument("--o", dest="ofile", help="output (TSV)")
  parser.add_argument("--rqfile", help="input Sparql file")
  parser.add_argument("--rq", help="input Sparql string")
  parser.add_argument("-v", "--verbose", action="count", default=0)
  args = parser.parse_args()

  logging.basicConfig(format="%(levelname)s:%(message)s", level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  logging.debug(verstr)

  fout = open(args.ofile, "w") if args.ofile else sys.stdout

  if args.op == "test":
    wikidata.Test(fout)

  elif args.op == "list_drugTargetPairs":
    wikidata.ListDrugTargetPairs(fout)

  elif args.op == "list_geneDiseasePairs":
    wikidata.ListGeneDiseasePairs(fout)

  else:
    parser.error(f"Unknown operation: {args.op}")



#!/usr/bin/env python3
#############################################################################
### See: http://www.brain-map.org/api/index.html
### http://help.brain-map.org/display/api/RESTful+Model+Access+%28RMA%29
### 
###  http://api.brain-map.org/api/v2/data/[Model]/[Model.id].[json|xml|csv]
###
###  http://api.brain-map.org/api/v2/data/Organism/1.xml
###  http://api.brain-map.org/api/v2/data/Gene/15.xml
###  http://api.brain-map.org/api/v2/data/Chromosome/12.json
###  http://api.brain-map.org/api/v2/data/Structure/4005.xml
###
### http://api.brain-map.org/api/v2/data/Organism/query.json
### http://api.brain-map.org/api/v2/data/Gene/describe.json
### http://api.brain-map.org/api/v2/data/enumerate.json
###
###  http://api.brain-map.org/api/v2/data/Gene/
###  http://api.brain-map.org/api/v2/data/Gene/18376.json
### 
###  &num_rows=[#]&start_row=[#]&order=[...]
#############################################################################
import sys,os,re,time,logging
import urllib.parse,json

from ...util import rest
#
##############################################################################
def ShowInfo(base_url, fout):
  rval = rest.Utils.GetURL(base_url+'/data/enumerate.json', parse_json=True)
  print(json.dumps(rval, sort_keys=True, indent=2))


##############################################################################
def ListProbes(base_url, qrys, fout):
  n_probe=0;
  num_rows=0; start_row=0; total_rows=0;
  for qry in qrys:
    while True:
      logging.debug('start_row = %d, num_rows = %d, total_rows = %d'%(start_row, num_rows, total_rows))
      url_this = (base_url+"/data/query.json?start_row=%d&criteria=model::Probe,rma::criteria,gene[acronym$eq'%s']"%(start_row+num_rows, qry))
      try:
        rval = rest.Utils.GetURL(url_this, parse_json=True)
      except Exception as e:
        logging.error(e)
        continue
      success = rval['success'] if 'success' in rval else False
      if not success:
        break
      id_this = rval['id'] if 'id' in rval else None
      total_rows = rval['total_rows'] if 'total_rows' in rval else None
      num_rows = rval['num_rows'] if 'num_rows' in rval else None
      start_row = rval['start_row'] if 'start_row' in rval else None
      probes = rval['msg'] if 'msg' in rval else []
      for probe in probes:
        n_probe+=1
        if n_probe==1:
          tags = sorted(probe.keys()) ##1st probe defines fields
          fout.write('%s\n'%('\t'.join(['query','id']+tags)))
        vals = [qry, id_this]+[(probe[tag] if tag in probe else '') for tag in tags]
        fout.write('%s\n'%('\t'.join(vals)))
      if start_row+num_rows >= total_rows:
        break
  logging.info('probes: %d'%n_probe)

##############################################################################



#!/usr/bin/env python3
"""
GWAS Catalog REST API client.

https://www.ebi.ac.uk/gwas/docs/api
https://www.ebi.ac.uk/gwas/rest/api
https://www.ebi.ac.uk/gwas/rest/docs/api
https://www.ebi.ac.uk/gwas/rest/docs/sample-scripts
"""
import sys,os,re,argparse,json,time,logging
#
from .. import gwascatalog
#
##############################################################################
if __name__=='__main__':
  epilog = """\
Example PMIDs: 28530673;
Example GCSTs: GCST004364, GCST000227;
Example EFOIDs: EFO_0004232;
Example SNPIDs: rs6085920, rs2273833, rs6684514, rs144991356
"""
  parser = argparse.ArgumentParser(description='GWAS Catalog REST API client', epilog=epilog)
  searchtypes=['pubmedmid', 'gcst', 'efotrait', 'efouri', 'accessionid', 'rs']
  ops = ['list_studies', 'search_studies', 'get_studyAssociations', 'get_snps']
  parser.add_argument("op", choices=ops, help='operation')
  parser.add_argument("--ids", dest="ids", help="IDs, comma-separated")
  parser.add_argument("--searchtype", choices=searchtypes, help="ID type")
  parser.add_argument("--i", dest="ifile", help="input file, IDs")
  parser.add_argument("--o", dest="ofile", help="output (TSV)")
  parser.add_argument("--skip", type=int, default=0)
  parser.add_argument("--nmax", type=int, default=None)
  parser.add_argument("--api_host", default=gwascatalog.API_HOST)
  parser.add_argument("--api_base_path", default=gwascatalog.API_BASE_PATH)
  parser.add_argument("-v", "--verbose", default=0, action="count")
  parser.add_argument("-q", "--quiet", default=0, action="count")
  args = parser.parse_args()

  base_url = 'https://'+args.api_host+args.api_base_path

  fout = open(args.ofile, 'w') if args.ofile else sys.stdout

  # logging.PROGRESS = 15 (custom)
  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>0 else logging.ERROR if args.quiet else 15))

  t0 = time.time()

  ids=[];
  if args.ifile:
    fin = open(args.ifile)
    while True:
      line = fin.readline()
      if not line: break
      ids.append(line.strip())
  elif args.ids:
    ids = re.split(r'\s*,\s*', args.ids.strip())
  if ids: logging.info(f"Input IDs: {len(ids)}")

  if args.op == 'search_studies':
    gwascatalog.SearchStudies(ids, args.searchtype, base_url, fout)

  elif args.op == 'list_studies':
    gwascatalog.ListStudies(base_url, fout)

  elif args.op == 'get_studyAssociations':
    gwascatalog.GetStudyAssociations(ids, args.skip, args.nmax, base_url, fout)

  elif args.op == 'get_snps':
    gwascatalog.GetSnps(ids, args.skip, args.nmax, base_url, fout)

  else:
    parser.error(f"Unknown operation: {args.op}")

  logging.info('{0}: elapsed time: {1}'.format(os.path.basename(sys.argv[0]), time.strftime('%Hh:%Mm:%Ss', time.gmtime(time.time()-t0))))




#!/usr/bin/env python3
"""
MeSH XML utility functions.

MeSH XML
Download: https://www.nlm.nih.gov/mesh/download_mesh.html
Doc: https://www.nlm.nih.gov/mesh/xml_data_elements.html
"""
###
import sys,os,re,logging
#import pandas as pd
import xml.etree.cElementTree as ElementTree
from xml.parsers import expat

#############################################################################
def Desc2Csv(branch, fin, fout):
  fout.write("id\ttreenum\tterm\n")
  n_elem=0; n_term=0;
  for event, elem in ElementTree.iterparse(fin):
    meshid,meshterm,desclass,treenum = None,None,None,None;
    n_elem+=1
    logging.debug(f"""Event:{event:6s}, elem.tag:{elem.tag:6s}, elem.text:'{(elem.text.strip() if elem.text else ''):6s}'""")
    if elem.tag == "DescriptorRecord":
      if elem.attrib["DescriptorClass"] != "1": continue
      meshid,meshterm,desclass,treenum = None,None,None,None;
      meshid=elem.findtext("DescriptorUI")
      #treenum=elem.findtext("TreeNumberList/TreeNumber")
      elems = elem.findall("TreeNumberList/TreeNumber")
      treenums = map(lambda e: e.text, elems)
      for treenum in treenums:
        if (re.match(f"^{branch}", treenum)): break
      meshterm=elem.findtext("DescriptorName/String")
      #See also: ConceptList/Concept/TermList/Term/String (may be multiple)
      if not meshid:
        logging.info(f"skipping, no meshid: {elem}")
      elif not meshterm:
        logging.info(f"meshid: {meshid} ; skipping, no meshterm")
      elif not treenum:
        logging.info(f"meshid: {meshid} ; skipping, no treenum")
      elif not re.match(f"^{branch}", treenum):
        logging.info(f"meshid: {meshid} ; skipping, non-{branch} treenum: {treenum}")
      else:
        fout.write(f"{meshid}\t{treenum}\t{meshterm}\n")
        n_term+=1
  logging.info(f"n_elem: {n_elem}")
  logging.info(f"n_term: {n_term}")

#############################################################################
### SCRClass
### Description: Attribute of <DescriptorRecord> one of:
### 1 = Regular chemical, drug, or other substance (the most common type)
### 2 = Protocol, for example, FOLFOXIRI protocol
### 3 = Rare disease, for example, Canicola fever
### Subelement of: not applicable. Attribute of <SupplementalRecord>
### Record Type: SCR
### https://www.nlm.nih.gov/mesh/xml_data_elements.html
#############################################################################
### There are non-disease records mapped to diseases, e.g. C041229,
### "GHM protein, human", SCRClass=1, is mapped to D006362, "Heavy Chain Disease".
### These cannot be identified from the supplementary file alone.
#############################################################################
def Supp2Csv(branch, fin, fout):
  fout.write("id\tterm\tid_to\tterm_to\n")
  n_elem=0; n_term=0;
  for event,elem in ElementTree.iterparse(fin):
    meshid,name,meshid_to,meshterm_to = None,None,None,None;
    n_elem+=1
    logging.debug(f"""Event:{event:6s}, elem.tag:{elem.tag:6s}, elem.text:'{(elem.text.strip() if elem.text else ''):6s}'""")
    if elem.tag == "SupplementalRecord":
      scrclass = elem.attrib["SCRClass"]
      if branch not in ("C", "D"): continue
      if branch=="C" and scrclass!="3": continue #disease-only
      if branch=="D" and scrclass!="1": continue #chemical-only
      meshid=elem.findtext("SupplementalRecordUI")
      name=elem.findtext("SupplementalRecordName/String")
      mtlist=elem.find("HeadingMappedToList")
      if mtlist is None:
        continue
      meshid_to=mtlist.findtext("HeadingMappedTo/DescriptorReferredTo/DescriptorUI")
      meshterm_to=mtlist.findtext("HeadingMappedTo/DescriptorReferredTo/DescriptorName/String")
      fout.write(f"{meshid}\t{name}\t{meshid_to}\t{meshterm_to}\n")
      n_term+=1
  logging.info(f"n_elem: {n_elem}")
  logging.info(f"n_term: {n_term}")

#############################################################################



#!/usr/bin/env python3
"""
See: https://docs.gdc.cancer.gov/API/Users_Guide/
"""
###
import sys,os,re,argparse,time,json,logging
#
from .. import tcga
#
API_HOST='api.gdc.cancer.gov'
API_BASE_PATH=''
#
##############################################################################
if __name__=='__main__':
  parser = argparse.ArgumentParser(description='TCGA REST API client')
  ops = [ 'list_projects', 'list_cases', 'list_files', 'list_annotations' ]
  parser.add_argument("op", choices=ops, help='operation')
  parser.add_argument("--i", dest="ifile", help="input IDs")
  parser.add_argument("--ids", help="input IDs (comma-separated)")
  parser.add_argument("--o", dest="ofile", help="output (TSV)")
  parser.add_argument("--skip", type=int, default=0)
  parser.add_argument("--nmax", type=int, default=None)
  parser.add_argument("--api_host", default=API_HOST)
  parser.add_argument("--api_base_path", default=API_BASE_PATH)
  parser.add_argument("-v", "--verbose", default=0, action="count")
  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  base_url = 'https://'+args.api_host+args.api_base_path

  fout = open(args.ofile,"w") if args.ofile else sys.stdout

  t0=time.time()

  ids=[];
  if args.ifile:
    fin = open(args.ifile)
    while True:
      line = fin.readline()
      if not line: break
      if line.rstrip(): ids.append(line.rstrip())
    fin.close()
  elif args.ids:
    ids = re.split(r'[,\s]+', args.ids)
  logging.info('Input queries: %d'%(len(ids)))

  if args.op == "list_projects":
    tcga.ListProjects(base_url, args.skip, args.nmax, fout)

  elif args.op == "list_cases":
    tcga.ListCases(base_url, args.skip, args.nmax, fout)

  elif args.op == "list_files":
    tcga.ListFiles(base_url, args.skip, args.nmax, fout)

  elif args.op == "list_annotations":
    tcga.ListAnnotations(base_url, args.skip, args.nmax, fout)

  else:
    parser.error("Invalid operation: %s"%args.op)

  logging.info(('elapsed time: %s'%(time.strftime('%Hh:%Mm:%Ss', time.gmtime(time.time()-t0)))))



#!/usr/bin/env python3
"""
Utility for STRING Db REST API.

STRING = Search Tool for the Retrieval of Interacting Genes/Proteins

See: http://string-db.org/help/api/

http://[database]/[access]/[format]/[request]?[parameter]=[value]

database:
string-db.org
string.embl.de
stitch.embl.de
"""
###
import sys,os,re,json,argparse,time,logging
import urllib,urllib.request,urllib.parse
#
from .. import stringdb
#
##############################################################################
if __name__=='__main__':
  parser = argparse.ArgumentParser(description='STRING-DB REST API client utility',
        epilog="""\
Example protein IDs: DRD1 DRD1_HUMAN DRD2 DRD2_HUMAN ;
Example species: 9606 (human, via taxon identifiers, http://www.uniprot.org/taxonomy) ;
Image formats: PNG PNG_highres SVG ;
MAY BE DEPRECATED: getInteractors, getActions, getAbstracts
""")
  ops = ['getIds', 'getInteractionPartners',
	'getNetwork', 'getNetworkImage',
	'getEnrichment', 'getPPIEnrichment',
	'getInteractors', 'getActions', 'getAbstracts']
  parser.add_argument("op",choices=ops,help='operation')
  parser.add_argument("--id", dest="id", help="protein ID (ex:DRD1_HUMAN)")
  parser.add_argument("--ids", dest="ids", help="protein IDs, comma-separated")
  parser.add_argument("--idfile", dest="idfile", help="input file, protein IDs")
  parser.add_argument("--o", dest="ofile", help="output file")
  parser.add_argument("--species", help="taxon code, ex: 9606 (human)")
  parser.add_argument("--minscore", type=int, default=500, help="signifcance threshold 0-1000")
  parser.add_argument("--netflavor", choices=stringdb.NETWORK_FLAVORS, default='evidence', help="network flavor")
  parser.add_argument("--imgfmt", choices=stringdb.IMG_FMTS, default='image', help="image format")
  parser.add_argument("--api_host", default=stringdb.API_HOST)
  parser.add_argument("--api_base_path", default=stringdb.API_BASE_PATH)
  parser.add_argument("-v", "--verbose", action="count", default=0)
  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  base_url='https://'+args.api_host+args.api_base_path

  ids=[]
  if args.idfile:
    fin = open(args.idfile)
    while True:
      line = fin.readline()
      if not line: break
      ids.append(line.strip())
  elif args.ids:
    ids = re.split(r'\s*,\s*', args.ids.strip())
  elif args.id:
    ids=[args.id]

  fout = open(args.ofile, "w+") if args.ofile else sys.stdout

  t0=time.time()

  if args.op == 'getIds':
    if not ids: parser.error('PID[s] required.')
    stringdb.GetIds(ids, base_url, fout)

  elif args.op == 'getInteractors':
    if not ids: parser.error('ID[s] required.')
    stringdb.GetInteractors(ids, args.species, args.minscore, base_url, fout)

  elif args.op == 'getInteractionPartners':
    if not ids: parser.error('ID[s] required.')
    stringdb.GetInteractionPartners(ids, args.species, args.minscore, base_url, fout)

  elif args.op == 'getActions':
    if not ids: parser.error('ID[s] required.')
    stringdb.GetActions(ids, args.species, args.minscore, base_url, fout)

  elif args.op == 'getEnrichment':
    if not len(ids)>1: parser.error('IDs (2+) required.')
    stringdb.GetEnrichment(ids, args.species, args.minscore, base_url, fout)

  elif args.op == 'getPPIEnrichment':
    if not len(ids)>1: parser.error('IDs (2+) required.')
    stringdb.GetPPIEnrichment(ids, args.species, args.minscore, base_url, fout)

  elif args.op == 'getAbstracts':
    if not ids: parser.error('ID[s] required.')
    stringdb.GetAbstracts(ids, args.species, base_url, fout)

  elif args.op == 'getNetwork':
    if not ids: parser.error('ID required.')
    stringdb.GetNetwork(ids[0], args.species, args.minscore, args.netflavor, base_url, fout)

  elif args.op == 'getNetworkImage':
    if not ids: parser.error('ID required.')
    if not args.ofile: parser.error('--o OUTFILE required.')
    fout.close()
    fout = open(args.ofile, "wb+") #binary
    img = stringdb.GetNetworkImage(ids[0], args.species, args.minscore, args.netflavor, args.imgfmt, base_url)
    fout.write(img)
    fout.close()

  else:
    parser.print_help()

  logging.info(('elapsed time: %s'%(time.strftime('%Hh:%Mm:%Ss',time.gmtime(time.time()-t0)))))




#!/usr/bin/env python
"""
pug_aids2assays.py - retrieve bioassays by AID from PubChem/PUG (circa 2010)
"""
import sys,os,re,urllib,time,getopt,tempfile,gzip
import pug_utils

PROG=os.path.basename(sys.argv[0])

PUGURL='http://pubchem.ncbi.nlm.nih.gov/pug/pug.cgi'
POLL_WAIT=10
MAX_WAIT=600

#############################################################################
def QueryPug_fetchassays(aids,format='csv',verbose=0):
  qxml='''\
<PCT-Data><PCT-Data_input><PCT-InputData>
  <PCT-InputData_query><PCT-Query><PCT-Query_type><PCT-QueryType>
    <PCT-QueryType_bas><PCT-QueryAssayData>
'''
  if format=='xml':
    qxml+='<PCT-QueryAssayData_output value="assay-xml">1</PCT-QueryAssayData_output>'
  else:
    qxml+='<PCT-QueryAssayData_output value="csv">4</PCT-QueryAssayData_output>'
  qxml+='''\
        <PCT-QueryAssayData_aids><PCT-QueryUids><PCT-QueryUids_ids>
          <PCT-ID-List>
            <PCT-ID-List_db>pcassay</PCT-ID-List_db>
            <PCT-ID-List_uids>
'''
  for aid in aids:
    qxml+=('<PCT-ID-List_uids_E>%d</PCT-ID-List_uids_E>\n'%aid)
  qxml+='''\
            </PCT-ID-List_uids>
          </PCT-ID-List>
        </PCT-QueryUids_ids></PCT-QueryUids></PCT-QueryAssayData_aids>
      <PCT-QueryAssayData_dataset value="complete">0</PCT-QueryAssayData_dataset>
      <PCT-QueryAssayData_focus>
        <PCT-Assay-FocusOption>
          <PCT-Assay-FocusOption_group-results-by value="substance">4</PCT-Assay-FocusOption_group-results-by>
        </PCT-Assay-FocusOption>
      </PCT-QueryAssayData_focus>
    </PCT-QueryAssayData></PCT-QueryType_bas>
  </PCT-QueryType></PCT-Query_type></PCT-Query></PCT-InputData_query>
</PCT-InputData></PCT-Data_input></PCT-Data>
'''

  # <PCT-Assay-FocusOption_group-results-by value="compound">0</PCT-Assay-FocusOption_group-results-by>

  if verbose:
    print >>sys.stderr,'connecting %s...'%PUGURL
    print >>sys.stderr,'submitting query for %d bioassays...'%len(aids)
  f=pug_utils.UrlOpenTry(PUGURL,qxml)
  pugxml=f.read()
  f.close()
  status,reqid,url,qkey,wenv,error=pug_utils.ParsePugXml(pugxml)
  return status,reqid,url,error

#############################################################################
if __name__=='__main__':
  n_chunk=1000;
  usage='''
  %(PROG)s - fetch PubChem bioassay files (CSV or XML) for AIDs

  required:
  --i=<infile>   ... input IDs file
  or
  --aids=<IDs>   ... input IDs, comma-separated
  and
  --o=<outfile>  ... output file

  options:
  --format=<FMT> ... csv|xml [default via filename]
  --nmax=<N>     ... maximum N IDs
  --skip=<N>     ... skip 1st N IDs in file
  --n_chunk=<N>  ... IDs per PUG request [%(N_CHUNK)d]
  --gz           ... output gzipped [default via filename]
  --v            ... verbose
  --vv           ... very verbose
  --h            ... this help
'''%{'PROG':PROG,'N_CHUNK':n_chunk}

  ifile=None; iaids=None; ofile=None; verbose=0; skip=0; nmax=0;
  format=None; gz=False; 
  opts,pargs = getopt.getopt(sys.argv[1:],'',['h','v','vv','i=','in=',
  'aids=','o=','out=','fmt=','format=','gz','gz','skip=','nmax=','n=',
  'n_chunk='])
  if not opts: pug_utils.ErrorExit(usage)
  for (opt,val) in opts:
    if opt=='--h': pug_utils.ErrorExit(usage)
    elif opt in ('--i','--in'): ifile=val
    elif opt=='--aids': iaids=val
    elif opt in ('--o','--out'): ofile=val
    elif opt in ('--n','--nmax'): nmax=int(val)
    elif opt in ('--format','--fmt'): format=val
    elif opt in ('--gzip','--gz'): gz=True
    elif opt=='--skip': skip=int(val)
    elif opt=='--n_chunk': n_chunk=int(val)
    elif opt=='--vv': verbose=2
    elif opt=='--v': verbose=1
    else: pug_utils.ErrorExit('Illegal option: %s'%val)

  if not ofile:
    pug_utils.ErrorExit('--o required\n'+usage)
  if not (ifile or iaids):
    pug_utils.ErrorExit('--i or --aids required\n'+usage)
  if (ofile[-3:]=='.gz'):
    gz=True
    ext=re.sub('^.*\.','',ofile[:-3])
  else:
    ext=re.sub('^.*\.','',ofile)
  if not format: format=ext
  if format not in ('csv','xml'):
    pug_utils.ErrorExit('format "csv" or "xml" required\n'+usage)

  if ifile:
    faids=file(ifile)
    if not faids:
      pug_utils.ErrorExit('cannot open: %s\n%s'%(ifile,usage))
    i=0; aids=[];
    while True:
      line=faids.readline()
      if not line: break
      line=line.strip()
      if not line: continue
      i+=1
      if skip and i<=skip: continue
      if nmax and i>nmax: break
      try:
        field=re.sub('\s.*$','',line)	## may be addl field
        aid=int(field)
        aids.append(aid)
      except:
        print >>sys.stderr,'cannot parse aid: "%s"'%line
        continue
  else:
    buff=iaids
    try:
      buff=re.sub('\s','',buff)
      aids=map(lambda x:int(x),re.split(',',buff))
    except:
      pug_utils.ErrorExit('cannot parse "--aids %s"'%iaids)
  print >>sys.stderr,'aids read: %d'%len(aids)

  if not gz:
    fd,ofile_tmp=tempfile.mkstemp('.'+format+'.gz',PROG)
    os.close(fd)
    fout=file(ofile_tmp,'w')
  else:
    fout=file(ofile,'w')

  if not fout:
    pug_utils.ErrorExit('cannot open: %s\n%s'%(ofile,usage))

  n=0; i_query=0; nbytes_total=0; n_query_total=0;
  while n<len(aids):
    i_query+=1
    n_query=len(aids[n:n+n_chunk])
    print >>sys.stderr,'Q%d. [%d-%d] of %d'%(i_query,n+1,n+n_query,len(aids))

    status,reqid,url,error=QueryPug_fetchassays(aids[n:n+n_chunk],format,verbose)
    pug_utils.CheckStatus(status,error)

    if verbose:
      print >>sys.stderr,'query submit status: %s'%status

    if not url and not reqid:
      pug_utils.ErrorExit('query submit ok but no ID; quitting.')

    i_poll=0
    while not url:
      if i_poll*POLL_WAIT>MAX_WAIT:
        pug_utils.PollPug(reqid,'cancel',verbose)
        pug_utils.ErrorExit('max wait exceeded (%d sec); quitting.'%MAX_WAIT)
      print >>sys.stderr,'polling PUG [ID=%d]...'%reqid
      status,reqid,url,qkey,wenv,error = pug_utils.PollPug(reqid,'status',verbose)
      pug_utils.CheckStatus(status,error)
      if url: break
  
      if verbose:
        print >>sys.stderr,'%d sec wait (status=%s)...'%(POLL_WAIT,status)
      time.sleep(POLL_WAIT)
      i_poll+=1

    if verbose:
      print >>sys.stderr,'URL: %s'%url
      print >>sys.stderr,'downloading to %s...'%ofile
    nbytes=pug_utils.DownloadUrl(url,fout)
    nbytes_total+=nbytes;
    print >>sys.stderr,'assays: %d (%.2fMB)'%(n_query,nbytes/1e6)
    n_query_total+=n_query
    n+=n_chunk

  fout.close()
  if not gz:
    ftmp=gzip.open(ofile_tmp,'rb')
    fout=file(ofile,'w')
    fout.write(ftmp.read())
    ftmp.close()
    fout.close()
    os.unlink(ofile_tmp)

  print >>sys.stderr,'%s: total assays: %d (%.2fMB)'%(PROG,n_query_total,nbytes_total/1e6)
  sys.stderr.write('%s: format: %s'%(PROG,format))
  if gz: sys.stderr.write('(gzipped)')
  sys.stderr.write('\n')



#!/usr/bin/env python3
"""
IDG Resource Submission System
https://rss.ccs.miami.edu/
"""
###
import sys,os,re,argparse,time,logging
#
from ...idg import rss
#
##############################################################################
if __name__=='__main__':
  epilog="""\
"""
  parser = argparse.ArgumentParser(description="IDG RSS (Resource Submission System) REST API client)", epilog=epilog)
  ops = [ "list_targets", "get_target_resources" ]
  parser.add_argument("op", choices=ops, help="operation")
  parser.add_argument("--i", dest="ifile", help="input IDs")
  parser.add_argument("--ids", help="IDs (comma-separated)")
  parser.add_argument("--o", dest="ofile", help="output (TSV)")
  parser.add_argument("--api_host", default=rss.Utils.API_HOST)
  parser.add_argument("--api_base_path", default=rss.Utils.API_BASE_PATH)
  parser.add_argument("-v", "--verbose", default=0, action="count")
  args = parser.parse_args()

  logging.basicConfig(format='%(levelname)s:%(message)s', level=(logging.DEBUG if args.verbose>1 else logging.INFO))

  fout = open(args.ofile, "w") if args.ofile else sys.stdout

  ids=[]
  if args.ifile:
    fin = open(args.ifile)
    while True:
      line = fin.readline()
      if not line: break
      ids.append(line.rstrip())
    fin.close()
  elif args.ids:
    ids = re.split('[,\s]+', args.ids.strip())
  if ids: logging.info('Input IDs: %d'%(len(ids)))

  t0=time.time()

  base_url = 'https://'+args.api_host+args.api_base_path

  if args.op=='list_targets':
    rss.Utils.ListTargets(base_url, fout)

  elif args.op=='get_target_resources':
    if not ids: parser.error(f"--i or --ids required for {args.op}")
    rss.Utils.GetTargetResources(ids, base_url, fout)

  else:
    parser.error(f'Unknown operation: {args.op}')

  logging.info(('elapsed time: %s'%(time.strftime('%Hh:%Mm:%Ss',time.gmtime(time.time()-t0)))))



#! /usr/bin/env python3
"""
Developed and tested with doid.obo (Disease Ontology)
"""
import sys,os,argparse,re,logging

#############################################################################
def OBO2CSV(fin, fout):
  n_in=0; n_rec=0; n_out=0;
  tags = ['id', 'name', 'namespace', 'alt_id', 'def', 'subset', 'synonym', 'xref', 'is_a', 'is_obsolete']
  fout.write('\t'.join(tags)+'\n')
  reclines=[];
  while True:
    line=fin.readline()
    if not line: break
    n_in+=1
    line=line.strip()
    if reclines:
      if line == '':
        row = OBO2CSV_Record(reclines)
        n_rec+=1
        vals=[]
        is_obsolete=False
        for tag in tags:
          if tag in row:
            val=row[tag]
            if tag in ('def', 'synonym'):
              val=re.sub(r'^"([^"]*)".*$', r'\1', val)
            else:
              val=re.sub(r'^"(.*)"$', r'\1', val)
          else:
            val=''
          if tag=='is_obsolete': is_obsolete = bool(val.lower() == "true")
          vals.append(val)
        if not is_obsolete:
          fout.write('\t'.join(vals)+'\n')
          n_out+=1
        reclines=[];
      else:
        reclines.append(line)
    else:
      if line == '[Term]':
        reclines.append(line)
      else: continue

  logging.info("input lines: %d; input records: %d ; output lines: %d"%(n_in, n_rec, n_out))

#############################################################################
def OBO2CSV_Record(reclines):
  vals={};
  if reclines[0] != '[Term]':
    logging.error('reclines[0] = "%s"'%reclines[0])
    return
  for line in reclines[1:]:
    line = re.sub(r'\s*!.*$','',line)
    k,v = re.split(r':\s*', line, maxsplit=1)
    if k=='xref' and not re.match(r'\S+:\S+$',v): continue
    if k not in vals: vals[k]=''
    vals[k] = '%s%s%s'%(vals[k],(';' if vals[k] else ''),v)
  return vals

#############################################################################
